import os
import shutil

from leapp import reporting
from leapp.actors import Actor
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api
from leapp.reporting import create_report, Report
from leapp.tags import FinalizationPhaseTag, IPUWorkflowTag


class CreateSystemdResumeService(Actor):
    """
    Add a systemd service to launch Leapp.

    Create a systemd service which will resume Leapp upgrade after the first reboot.
    """

    name = 'create_systemd_service'
    consumes = ()
    produces = (Report,)
    tags = (FinalizationPhaseTag, IPUWorkflowTag)

    def process(self):
        service_name = 'leapp_resume.service'
        systemd_dir = '/etc/systemd/system'

        service_templ_fpath = self.get_file_path(service_name)
        shutil.copyfile(service_templ_fpath, os.path.join(systemd_dir, service_name))

        service_path = '/etc/systemd/system/{}'.format(service_name)
        symlink_path = '/etc/systemd/system/default.target.wants/{}'.format(service_name)

        # in case nothing is enabled in the default target, the directory does not exist
        try:
            os.mkdir(os.path.join(systemd_dir, 'default.target.wants'))
        except OSError:
            pass

        if os.path.exists(symlink_path):
            api.current_logger().debug(
                'Symlink {} already exists (from previous upgrade?). Removing... '.format(symlink_path)
            )
            os.unlink(symlink_path)

        try:
            os.symlink(service_path, symlink_path)
        except OSError as e:
            raise StopActorExecutionError(
                    'Could not create a symlink to enable {}'.format(service_name),
                    details={"details": str(e)})

        create_report([
            reporting.Title('Leapp resume systemd service enabled'),
            reporting.Summary(
                '{} enabled as oneshot systemd service to resume Leapp execution '
                'after reboot.'.format(service_name)
            ),
            reporting.Severity(reporting.Severity.INFO),
            reporting.Groups([reporting.Groups.UPGRADE_PROCESS]),
            reporting.RelatedResource('file', service_path),
            reporting.RelatedResource('file', symlink_path),
            reporting.RelatedResource('service', service_name)
        ])



import six

from leapp.libraries.stdlib import api

ARCH_X86_64 = 'x86_64'
ARCH_ARM64 = 'aarch64'
ARCH_PPC64LE = 'ppc64le'
ARCH_S390X = 's390x'
ARCH_ACCEPTED = (ARCH_X86_64, ARCH_ARM64, ARCH_PPC64LE, ARCH_S390X)
ARCH_SUPPORTED = (ARCH_X86_64, ARCH_ARM64, ARCH_PPC64LE, ARCH_S390X)


def matches_architecture(*match_list):
    """
    Check if one of provided architectures matches the system's one.

    :param match_list: specification of architectures to check against
    :type match_list: strings
    :return: ``True`` if system's architecture matches one of the values in match_list, ``False`` otherwise
    :rtype: bool
    """
    if not all(isinstance(e, six.string_types) for e in match_list):
        raise TypeError("Architectures to check against have to strings "
                        "but provided was {}: '{}'".format([type(e) for e in match_list], match_list))
    unsupported = set(match_list).difference(ARCH_ACCEPTED)
    if unsupported:
        api.current_logger().warning("Unsupported architecture specified: {}".format(unsupported))
    return api.current_actor().configuration.architecture in match_list



import leapp.libraries.common.spamassassinutils as lib


def test_parse_sysconfig_spamassassin_begins_with_assignment():
    content = 'SPAMDOPTIONS="foo"\n# bar\n'
    pre, assignment, post = lib.parse_sysconfig_spamassassin(content)
    assert pre == ''
    assert assignment == 'SPAMDOPTIONS="foo"'
    assert post == '# bar\n'


def test_parse_sysconfig_spamassassin_ends_with_assignment():
    content = '# bar\nSPAMDOPTIONS="foo"\n'
    pre, assignment, post = lib.parse_sysconfig_spamassassin(content)
    assert pre == '# bar'
    assert assignment == 'SPAMDOPTIONS="foo"'
    assert post == ''


def test_parse_sysconfig_spamassassin_only_assignment():
    content = 'SPAMDOPTIONS="foo"\n'
    pre, assignment, post = lib.parse_sysconfig_spamassassin(content)
    assert pre == ''
    assert assignment == 'SPAMDOPTIONS="foo"'
    assert post == ''


def test_parse_sysconfig_spamassassin_no_assignment():
    content = '# foo\n'
    pre, assignment, post = lib.parse_sysconfig_spamassassin(content)
    assert pre == '# foo\n'
    assert assignment == ''
    assert post == ''


def test_parse_sysconfig_spamassassin_empty():
    content = ''
    pre, assignment, post = lib.parse_sysconfig_spamassassin(content)
    assert pre == ''
    assert assignment == ''
    assert post == ''



import os.path
from collections import namedtuple

import pytest

from leapp import reporting
from leapp.exceptions import StopActorExecution
from leapp.libraries.actor.pes_event_parsing import (
    Action,
    Event,
    get_pes_events,
    Package,
    parse_entry,
    parse_packageset,
    parse_pes_events
)
from leapp.libraries.common import fetch
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import ConsumedDataAsset

CUR_DIR = os.path.dirname(os.path.abspath(__file__))


def test_parse_packageset(current_actor_context):
    pkgset = {'package': [{'name': 'pkg1', 'repository': 'Repo'}]}

    parsed = parse_packageset(pkgset)
    assert len(parsed) == 1
    assert Package('pkg1', 'Repo', (None,)) in parsed

    assert parse_packageset({}) == set()
    assert parse_packageset({'set_id': 0}) == set()


def test_parse_packageset_modular(current_actor_context):
    modulestreams = [{'name': 'hey', 'stream': 'lol1'}, {'name': 'hey', 'stream': 'lol2'}]
    pkgset = {'package': [{'name': 'pkg1', 'repository': 'Repo', 'modulestreams': [None]},
                          {'name': 'pkg2', 'repository': 'Repo', 'modulestreams': modulestreams}]}

    parsed = parse_packageset(pkgset)
    assert len(parsed) == 2
    assert Package('pkg1', 'Repo', (None,)) in parsed
    assert Package('pkg2', 'Repo', (('hey', 'lol1'), ('hey', 'lol2'))) in parsed

    assert parse_packageset({}) == set()
    assert parse_packageset({'set_id': 0}) == set()


PARSE_ENTRY_INPUTS = [
    (
        # Input
        {
            'action': 4,
            'architectures': ['x86_64', 's390x'],
            'in_packageset': {
                'package': [{'name': 'original', 'repository': 'repo'}]
            },
            'out_packageset': {
                'package': [{'name': 'split01', 'repository': 'repo'}, {'name': 'split02', 'repository': 'repo'}]
            }
        },
        # Expected output
        [
            Event(action=Action.SPLIT, architectures=['x86_64', 's390x'], from_release=(0, 0), to_release=(0, 0), id=0,
                  in_pkgs={Package('original', 'repo', None)},
                  out_pkgs={Package('split01', 'repo', None), Package('split02', 'repo', None)})
        ]
    ),
    (
        # Input
        {
            'action': 1,
            'in_packageset': {
                'package': [{'name': 'removed', 'repository': 'repo'}]
            }
        },
        # Expected output
        [
            Event(action=Action.REMOVED, architectures=[], from_release=(0, 0), to_release=(0, 0), id=0,
                  in_pkgs={Package('removed', 'repo', None)},
                  out_pkgs=set())
        ]
    ),
    ({'action': 10}, ValueError),  # Invalid action
    ({'action': -1}, ValueError),  # Invalid action
    (
        {'action': 1, 'architectures': ['ia-64']},  # Invalid architecture
        ValueError
    ),
]


@pytest.mark.parametrize(('pes_entry_data', 'expected_output'), PARSE_ENTRY_INPUTS)
def test_parse_entry(current_actor_context, pes_entry_data, expected_output):
    """
    Tests whether the PES event is correctly parsed from the supplied dictionary with the same
    structure as are the data stored inside the json.
    """

    if isinstance(expected_output, list):
        events = parse_entry(pes_entry_data)
        assert len(events) == len(expected_output)
        for event in events:
            assert event in expected_output
    else:
        with pytest.raises(expected_output):
            parse_entry(pes_entry_data)


def test_parse_pes_events(current_actor_context):
    """
    Tests whether all events are correctly parsed from the provided string with the JSON data.
    """
    with open(os.path.join(CUR_DIR, 'files/sample01.json')) as f:
        events = parse_pes_events(f.read())
    assert len(events) == 2
    assert events[0].action == Action.SPLIT
    assert events[0].in_pkgs == {Package('original', 'repo', None)}
    assert events[0].out_pkgs == {Package('split01', 'repo', None), Package('split02', 'repo', None)}
    assert events[1].action == Action.REMOVED
    assert events[1].in_pkgs == {Package('removed', 'repo', None)}
    assert events[1].out_pkgs == set()


def test_parse_pes_events_with_modulestreams(current_actor_context):
    """
    Tests whether all events are correctly parsed from the provided string with the JSON data.
    """
    with open(os.path.join(CUR_DIR, 'files/sample04.json')) as f:
        events = parse_pes_events(f.read())
    assert len(events) == 5
    Expected = namedtuple('Expected', 'action,in_pkgs,out_pkgs')
    expected = [
        Expected(action=Action.SPLIT, in_pkgs={Package('original', 'repo', ('module', 'stream_in'))}, out_pkgs={
                 Package('split01', 'repo', None), Package('split02', 'repo', None)}),
        Expected(action=Action.SPLIT, in_pkgs={Package('original', 'repo', None)},
                 out_pkgs={Package('split01', 'repo', ('module', 'stream_out')),
                           Package('split02', 'repo', ('module', 'stream_out'))}),
        Expected(action=Action.REMOVED, in_pkgs={Package('removed', 'repo', None)}, out_pkgs=set()),
        Expected(action=Action.RENAMED, in_pkgs={Package('modularized', 'repo', ('module', 'stream_in'))}, out_pkgs={
                 Package('demodularized', 'repo', None)}),
        Expected(action=Action.RENAMED, in_pkgs={Package('modularized', 'repo', None)}, out_pkgs={
                 Package('demodularized', 'repo', None)}),
    ]

    for event in events:
        for idx, expectation in enumerate(list(expected)):
            if expectation.action == event.action and expectation.in_pkgs == event.in_pkgs:
                assert event.out_pkgs == expectation.out_pkgs
                expected.pop(idx)
                break
        if not expected:
            break
    assert not expected


def test_get_pes_events_invalid_data_reported(monkeypatch):
    def load_data_asset_mocked(*args, **kwargs):
        raise ValueError()

    monkeypatch.setattr(fetch, 'load_data_asset', load_data_asset_mocked)
    created_reports = create_report_mocked()
    monkeypatch.setattr(reporting, "create_report", created_reports)
    monkeypatch.setattr(api, "current_actor", CurrentActorMocked())

    with pytest.raises(StopActorExecution):
        get_pes_events("doesn't", "matter")

    assert created_reports.called



from leapp.actors import Actor
from leapp.libraries.actor import sctpupdate
from leapp.models import SCTPConfig
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag


class SCTPConfigUpdate(Actor):
    """
    Updates the kernel module blacklist for SCTP.

    If the SCTP module is wanted on RHEL8 the modprobe configuration gets updated to remove SCTP from the black listed
    kernel modules.
    """
    name = 'sctp_config_update'
    description = 'This actor updates SCTP configuration for RHEL8.'
    consumes = (SCTPConfig,)
    produces = ()
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)

    def process(self):
        sctpupdate.perform_update()



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class VdoConversionDevice(Model):
    topic = SystemInfoTopic
    name = fields.String()


class VdoConversionPreDevice(VdoConversionDevice):
    pass


class VdoConversionFailiableDevice(VdoConversionDevice):
    # `check_failed` and `failure` are only set if the checking process fails.
    check_failed = fields.Boolean(default=False)
    failure = fields.Nullable(fields.String())


class VdoConversionPostDevice(VdoConversionFailiableDevice):
    complete = fields.Boolean()


class VdoConversionUndeterminedDevice(VdoConversionFailiableDevice):
    # There are only devices which are undetermined as to VDO conversion if
    # lvm is installed on the system and either vdo is not or
    # `vdoprepareforlvm` fails.
    #
    # If the vdo package is not installed on the system `check_failed` is
    # false (the device wasn't actually checked); this indicates the absence of
    # the vdo package on the system.
    pass


class VdoConversionInfo(Model):
    # If lvm is not installed on the system there can be no VDO instances.
    # In that case VdoConversionInfo is generated with empty lists of devices.
    topic = SystemInfoTopic

    pre_conversion = fields.List(fields.Model(VdoConversionPreDevice), default=[])
    post_conversion = fields.List(fields.Model(VdoConversionPostDevice), default=[])
    undetermined_conversion = fields.List(fields.Model(VdoConversionUndeterminedDevice), default=[])



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class RootSubdirectory(Model):
    """
    Representation of a single root subdirectory. Can be expanded as needed.
    """
    topic = SystemFactsTopic
    name = fields.String()
    target = fields.Nullable(fields.String())  # if it's a link


class InvalidRootSubdirectory(Model):
    """
    Representation of a single root subdirectory with non-utf name that is stored as a Blob.
    """
    topic = SystemFactsTopic
    name = fields.Blob()
    target = fields.Nullable(fields.Blob())


class RootDirectory(Model):
    topic = SystemFactsTopic
    items = fields.List(fields.Model(RootSubdirectory))  # should not be empty
    invalid_items = fields.Nullable(fields.List(fields.Model(InvalidRootSubdirectory)))



import glob
import os

from leapp.libraries.stdlib import api


def remove_sssd_cache_files(config):
    """
    Remove all SSSD cache files.
    """
    if not config:
        return

    for cache_file in glob.glob('/var/lib/sss/db/*.ldb'):
        try:
            api.current_logger().info('Pruning SSSD cache file {}'.format(cache_file))
            os.remove(cache_file)
        except OSError:
            api.current_logger().warning('Failed to remove cache file: %s', cache_file)



from leapp import reporting
from leapp.actors import Actor
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.actor.tcpwrapperscheck import config_affects_daemons
from leapp.libraries.common.rpms import create_lookup
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM, Report, TcpWrappersFacts
from leapp.reporting import create_report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag

DAEMONS = [
    ("audit", ["auditd"]),
    ("bacula", ["bacula"]),
    ("conserver", ["conserver"]),
    ("cyrus-imapd", ["imap", "pop3", "imaps", "pop3s", "sieve", "lmtp"]),
    ("dovecot", ["imap", "pop3", "imaps", "pop3s", "sieve", "lmtp"]),
    ("nfs-utils", ["mountd", "statd"]),
    ("openssh-server", ["sshd"]),
    ("proftpd", ["proftpd"]),
    ("quota", ["rquotad"]),
    ("rpcbind", ["rpcbind"]),
    # ("sendmail", ["sendmail"]),  # Handled in CheckSendmail Actor
    ("slapi-nis", ["slapi-nis"]),
    ("socat", ["socat"]),
    ("stunnel", ["stunnel"]),
    ("tftp-server", ["tftpd"]),
    # ("vsftpd", ["vsftpd"]),  # Handled in VsftpdConfigCheck Actor
    ("xinetd", ["xinetd"]),
]


class TcpWrappersCheck(Actor):
    """
    Check the list of packages previously compiled with TCP wrappers support
    and check whether they have some rules configured in
    /etc/hosts.{allow,deny} that are no longer honored by the daemons and
    might make the service more accessible than expected.
    """

    name = 'tcp_wrappers_check'
    consumes = (TcpWrappersFacts, DistributionSignedRPM,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        # Consume a single TCP Wrappers message
        tcp_wrappers_messages = self.consume(TcpWrappersFacts)
        tcp_wrappers_facts = next(tcp_wrappers_messages, None)
        if list(tcp_wrappers_messages):
            api.current_logger().warning('Unexpectedly received more than one TcpWrappersFacts message.')
        if not tcp_wrappers_facts:
            raise StopActorExecutionError(
                'Could not check tcp wrappers configuration', details={'details': 'No TcpWrappersFacts found.'}
            )

        # Convert installed packages message to list
        packages = create_lookup(DistributionSignedRPM, field='items', keys=('name',))

        found_packages = config_affects_daemons(tcp_wrappers_facts, packages, DAEMONS)

        if found_packages:
            create_report([
                reporting.Title('TCP Wrappers configuration affects some installed packages'),
                reporting.Summary(
                    'tcp_wrappers support has been removed in RHEL-8. '
                    'There is some configuration affecting installed packages (namely {}) '
                    'in /etc/hosts.deny or /etc/hosts.allow, which '
                    'is no longer going to be effective after update. '
                    'Please migrate it manually.'.format(', '.join(found_packages))
                ),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.ExternalLink(
                    title='Replacing TCP Wrappers in RHEL 8',
                    url='https://access.redhat.com/solutions/3906701'
                ),
                reporting.Groups([reporting.Groups.SECURITY, reporting.Groups.NETWORK]),
                reporting.Groups([reporting.Groups.INHIBITOR]),
                reporting.RelatedResource('file', '/etc/hosts.allow'),
                reporting.RelatedResource('file', '/etc/hosts.deny'),
                reporting.RelatedResource('package', 'tcp_wrappers')
            ] + [reporting.RelatedResource('package', fp) for fp in found_packages])



from leapp.models import SSSDConfig8to9


class SSSDFactsLibrary(object):
    """
    Helper library from SSSDFacts actor to allow unit testing.
    """
    def __init__(self, config):
        self.config = config

    def process(self):
        """
        Check SSSD configuration for the following options:

        - sssd/enable_files_domain
        - pam/pam_cert_auth
        - explicit files provider domain present
        """
        facts = SSSDConfig8to9(
            enable_files_domain_set=False,
            explicit_files_domain=False,
            pam_cert_auth=False,
        )

        facts.enable_files_domain_set = self.config.has_option('sssd', 'enable_files_domain')
        facts.pam_cert_auth = self.config.getboolean('pam', 'pam_cert_auth', fallback=False)

        # Lookup domain with id_provider == files
        for section in self.config.sections():
            # Not a domain config.
            if not section.startswith('domain/'):
                continue

            # Malformed config?
            if not self.config.has_option(section, 'id_provider'):
                continue

            val = self.config.get(section, 'id_provider', fallback=None)
            if val == 'files':
                facts.explicit_files_domain = True
                break

        return facts



import pytest

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.actor import opensshprotocolcheck
from leapp.models import OpenSshConfig, OpenSshPermitRootLogin, Report
from leapp.snactor.fixture import current_actor_context


def test_no_config(current_actor_context):
    with pytest.raises(StopActorExecutionError):
        opensshprotocolcheck.process(iter([]))


osprl = OpenSshPermitRootLogin(value='no')


@pytest.mark.parametrize('protocol', [None, '1', '2', '1,2', '2,1', '7'])
def test_protocol(current_actor_context, protocol):
    current_actor_context.feed(OpenSshConfig(
        permit_root_login=[osprl],
        protocol=protocol,
        deprecated_directives=[]
    ))
    current_actor_context.run()
    if protocol:
        assert current_actor_context.consume(Report)
    else:
        assert not current_actor_context.consume(Report)



from leapp.libraries.stdlib import api
from leapp.models import PkgManagerInfo, RHUIInfo


def _set_releasever(releasever):
    releasever_path = '/etc/dnf/vars/releasever'

    with open(releasever_path, 'w') as fo:
        fo.write(releasever+'\n')


def process():
    target_version = api.current_actor().configuration.version.target

    pkg_facts = next(api.consume(PkgManagerInfo), None)
    rhui_facts = next(api.consume(RHUIInfo), None)

    if pkg_facts and pkg_facts.etc_releasever is not None or rhui_facts:
        # if "/etc/dnf/vars/releasever" file exists, or we are using RHUI, let's set it to our
        # target version.
        _set_releasever(target_version)
    else:
        api.current_logger().debug(
            'Skipping execution. "releasever" is not set in DNF/YUM vars directory and no RHUIInfo has '
            'been produced'
        )



from leapp.actors import Actor
from leapp.libraries.actor import scan_source_kernel as scan_source_kernel_lib
from leapp.models import DistributionSignedRPM, KernelInfo
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class ScanSourceKernel(Actor):
    """
    Scan the source system kernel.
    """

    name = 'scan_source_kernel'
    consumes = (DistributionSignedRPM,)
    produces = (KernelInfo,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        scan_source_kernel_lib.scan_source_kernel()



import os
import uuid
from argparse import Namespace

from leapp.cli.commands.upgrade import upgrade, util
from leapp.exceptions import CommandError
from leapp.utils.audit import Execution, get_connection
from leapp.utils.audit.contextclone import clone_context
from leapp.utils.clicmd import command, command_arg, command_opt

RERUN_SUPPORTED_PHASES = ('FirstBoot',)


@command('rerun', help='Re-runs the upgrade from the given phase and using the information and progress '
         'from the last invocation of leapp upgrade.')
@command_arg('from-phase',
             help='Phase to start running from again. Supported values: {}'.format(', '.join(RERUN_SUPPORTED_PHASES)))
@command_opt('only-actors-with-tag', action='append', metavar='TagName',
             help='Restrict actors to be re-run only with given tags. Others will not be executed')
@command_opt('debug', is_flag=True, help='Enable debug mode', inherit=False)
@command_opt('verbose', is_flag=True, help='Enable verbose logging', inherit=False)
def rerun(args):

    if os.environ.get('LEAPP_UNSUPPORTED') != '1':
        raise CommandError('This command requires the environment variable LEAPP_UNSUPPORTED="1" to be set!')

    if args.from_phase not in RERUN_SUPPORTED_PHASES:
        raise CommandError('This command is only supported for {}'.format(', '.join(RERUN_SUPPORTED_PHASES)))

    context = str(uuid.uuid4())
    last_context, configuration = util.fetch_last_upgrade_context()
    phases = [chkpt['phase'] for chkpt in util.get_checkpoints(context=last_context)]
    if args.from_phase not in set(phases):
        raise CommandError('Phase {} has not been executed in the last leapp upgrade execution. '
                           'Cannot rerun not executed phase'.format(args.from_phase))

    if not last_context:
        raise CommandError('No previous upgrade run to rerun - '
                           'leapp upgrade has to be run before leapp rerun can be used')

    with get_connection(None) as db:
        e = Execution(context=context, kind='rerun', configuration=configuration)

        e.store(db)

        clone_context(last_context, context, db)
        db.execute('''
            DELETE FROM audit WHERE id IN (
                SELECT
                    audit.id          AS id
                FROM
                    audit
                JOIN
                    data_source ON data_source.id = audit.data_source_id
                WHERE
                    audit.context = ? AND audit.event = 'checkpoint'
                    AND data_source.phase LIKE 'FirstBoot%'
            );
        ''', (context,))
        db.execute('''DELETE FROM message WHERE context = ? and type = 'ErrorModel';''', (context,))

    util.archive_logfiles()
    upgrade(Namespace(  # pylint: disable=no-value-for-parameter
        resume=True,
        resume_context=context,
        only_with_tags=args.only_actors_with_tag or [],
        debug=args.debug,
        verbose=args.verbose,
        reboot=False,
        no_rhsm=False,
        nogpgcheck=False,
        channel=None,
        report_schema='1.1.0',
        whitelist_experimental=[],
        enablerepo=[]))


def register(base_command):
    """
        Registers `leapp rerun`
    """
    base_command.add_sub(rerun)



from leapp import reporting
from leapp.actors import Actor
from leapp.models import SSSDConfig
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag

COMMON_REPORT_TAGS = [reporting.Groups.AUTHENTICATION, reporting.Groups.SECURITY]

related = [
    reporting.RelatedResource('package', 'sssd'),
    reporting.RelatedResource('file', '/etc/sssd/sssd.conf')
]


class SSSDCheck(Actor):
    """
    Check SSSD configuration for changes in RHEL8 and report them.

    These changes are:
    - id_provider=local is no longer supported and will be ignored
    - ldap_groups_use_matching_rule_in_chain was removed and will be ignored
    - ldap_initgroups_use_matching_rule_in_chain was removed and will be ignored
    - ldap_sudo_include_regexp changed default from true to false
    """

    name = 'sssd_check'
    consumes = (SSSDConfig,)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        model = next(self.consume(SSSDConfig), None)
        if not model:
            return

        for domain in model.domains:
            if 'local_provider' in domain.options:
                self.reportLocalProvider(domain)

            if 'groups_chain' in domain.options:
                self.reportRemovedOption(domain, 'ldap_groups_use_matching_rule_in_chain')

            if 'initgroups_chain' in domain.options:
                self.reportRemovedOption(domain, 'ldap_initgroups_use_matching_rule_in_chain')

            if 'sudo_regexp' in domain.options:
                self.reportSudoRegexp(domain)

    def reportLocalProvider(self, domain):
        create_report([
            reporting.Title('SSSD Domain "%s": local provider is no longer '
                            'supported and the domain will be ignored.' % domain.name),
            reporting.Summary('Local provider is no longer supported.'),
            reporting.Groups(COMMON_REPORT_TAGS),
            reporting.Severity(reporting.Severity.MEDIUM)
        ] + related)

    def reportRemovedOption(self, domain, option):
        create_report([
            reporting.Title('SSSD Domain "%s": option %s has no longer '
                            'any effect' % (domain.name, option)),
            reporting.Summary('Option %s was removed and it will be ignored.' % option),
            reporting.Groups(COMMON_REPORT_TAGS),
            reporting.Severity(reporting.Severity.MEDIUM)
        ] + related)

    def reportSudoRegexp(self, domain):
        create_report([
            reporting.Title('SSSD Domain "%s": sudo rules containing wildcards '
                            'will stop working.' % domain.name),
            reporting.Summary('Default value of ldap_sudo_include_regexp changed '
                              'from true to false for performance reason.'),
            reporting.Groups(COMMON_REPORT_TAGS),
            reporting.Remediation(
                hint='If you use sudo rules with wildcards, set this option to true explicitly.'
            ),
            reporting.Severity(reporting.Severity.HIGH)
        ] + related)



from six.moves import configparser

from leapp.actors import Actor
from leapp.libraries.actor.sssdfacts import SSSDFactsLibrary
from leapp.models import SSSDConfig
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class SSSDFacts(Actor):
    """
    Check SSSD configuration for changes in RHEL8 and report them in model.

    These changes are:
    - id_provider=local is no longer supported and will be ignored
    - ldap_groups_use_matching_rule_in_chain was removed and will be ignored
    - ldap_initgroups_use_matching_rule_in_chain was removed and will be ignored
    - ldap_sudo_include_regexp changed default from true to false
    """

    name = 'sssd_facts'
    consumes = ()
    produces = (SSSDConfig,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        try:
            config = configparser.RawConfigParser()
            config.read('/etc/sssd/sssd.conf')
        except configparser.Error:
            # SSSD is not configured properly. Nothing to do.
            self.log.warning('SSSD configuration unreadable.')
            return

        self.produce(SSSDFactsLibrary(config).process())



import pytest

from leapp.libraries.common import persistentnetnames
from leapp.models import Interface, PCIAddress, PersistentNetNamesFactsInitramfs
from leapp.snactor.fixture import current_actor_context


def interface_mocked(i=0):
    return Interface(
        name='n{}'.format(i),
        devpath='dp{}'.format(i),
        driver='d{}'.format(i),
        vendor='v{}'.format(i),
        pci_info=PCIAddress(
            domain='pd{}'.format(i),
            bus='pb{}'.format(i),
            function='pf{}'.format(i),
            device='pd{}'.format(i)
        ),
        mac='m{}'.format(i)
    )


class interfaces_mocked(object):
    def __init__(self, count):
        self.count = count

    def __call__(self):
        for i in range(self.count):
            yield interface_mocked(i)


@pytest.mark.parametrize('count', [0, 1, 8, 256])
def test_run(monkeypatch, current_actor_context, count):
    monkeypatch.setattr(persistentnetnames, 'interfaces', interfaces_mocked(count))
    current_actor_context.run()
    assert len(current_actor_context.consume(PersistentNetNamesFactsInitramfs)[0].interfaces) == count



from leapp.actors import Actor
from leapp.libraries.actor.getinstalleddesktops import get_installed_desktops
from leapp.models import InstalledDesktopsFacts, InstalledRPM
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class GetInstalledDesktops(Actor):
    """
    Actor checks if kde or gnome desktop environments
    are installed and what desktop is default.
    """

    name = 'get_installed_desktops'
    consumes = (InstalledRPM,)
    produces = (InstalledDesktopsFacts,)
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        facts = get_installed_desktops()
        self.produce(InstalledDesktopsFacts(
            gnome_installed=facts["gnome_installed"],
            kde_installed=facts["kde_installed"]))



import pytest

from leapp import reporting
from leapp.libraries.actor import reportsettargetrelease
from leapp.libraries.common import rhsm
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api


@pytest.mark.parametrize('version', ['8.{}'.format(i) for i in range(4)])
def test_report_target_version(monkeypatch, version):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(dst_ver=version))
    monkeypatch.setattr(rhsm, 'skip_rhsm', lambda: False)
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    SUMMARY_FMT = 'will be set to {}.'
    reportsettargetrelease.process()
    assert reporting.create_report.called == 1
    assert SUMMARY_FMT.format(version) in reporting.create_report.report_fields['summary']
    assert 'is going to be set' in reporting.create_report.report_fields['title']


def test_report_unhandled_release(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(dst_ver='8.1'))
    monkeypatch.setattr(rhsm, 'skip_rhsm', lambda: True)
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    reportsettargetrelease.process()
    assert reporting.create_report.called == 1
    assert 'is going to be kept' in reporting.create_report.report_fields['title']



import os

from leapp.libraries.stdlib import api
from leapp.models import RpmTransactionTasks


def process():
    location = api.get_folder_path('bundled-rpms')
    local_rpms = []
    for name in os.listdir(location):
        if name.endswith('.rpm'):
            # It is important to put here the realpath to the files here, because
            # symlinks cannot be resolved properly inside of the target userspace since they use the /installroot
            # mount target
            local_rpms.append(os.path.realpath(os.path.join(location, name)))
    if local_rpms:
        api.produce(RpmTransactionTasks(local_rpms=local_rpms))



from leapp.actors import Actor
from leapp.libraries.actor import check_fips as check_fips_lib
from leapp.models import FIPSInfo
from leapp.tags import IPUWorkflowTag, LateTestsPhaseTag


class CheckFIPSCorrectlyEnabled(Actor):
    """
    Sanity check to stop the IPU if the system did not boot into the upgrade initramfs with FIPS settings preserved.

    The performed check should be unlikely to fail, as it would mean that the upgrade boot entry was created without
    fips=1 on the kernel cmdline.
    """

    name = 'check_fips_correctly_enabled'
    consumes = (FIPSInfo,)
    produces = ()
    tags = (LateTestsPhaseTag, IPUWorkflowTag)

    def process(self):
        check_fips_lib.check_fips_state_perserved()



import os

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.rpms import has_package
from leapp.libraries.stdlib import api
from leapp.models import CupsChangedFeatures, DistributionSignedRPM


def _list_dir(path):
    """
    Lists files which are in a directory specified by the path.

    :param str path: path to directory
    """
    return os.listdir(path)


def _path_exists(path):
    """
    Checks if the path exists on the machine.

    :param str path: path to file/directory
    """
    return os.path.exists(path)


def _read_file(path):
    """
    Read a file line by line.

    :param str path: path to file
    """
    with open(path, 'r') as f:
        return f.readlines()


def _check_package(pkg):
    """
    Checks if a package is installed and signed.

    :param str pkg: name of package
    """
    return has_package(DistributionSignedRPM, pkg)


def directive_exists(name, line):
    """
    Checks if directive exists in the line, but it is not
    commented out.

    :param str name: name of directive
    :param str line: line of file
    """
    return line.lstrip().startswith(name)


def get_directive_value(name, line):
    """
    Returns directive value.

    :param str name: name of directive
    :param str line: line of file
    """
    if directive_exists(name, line):
        return line.lstrip().lstrip(name).lstrip().split(' ')[0].rstrip()
    return None


def interface_script_check(check_path_func=_path_exists,
                           list_dir_func=_list_dir):
    """
    Checks if any file is in /etc/cups/interfaces, which means there could be
    print queues using interface script.

    :param func check_path_func: checks if /etc/cups/interfaces exists
    :param func list_dir_func: lists contents of directory
    """
    if (
            check_path_func('/etc/cups/interfaces') and
            list_dir_func('/etc/cups/interfaces')
       ):
        return True
    return False


def include_directive_check(read_func=_read_file):
    """
    Checks if 'Include' directive is present.

    :param str paths: path to cupsd configuration file
    :param func read_func: function for reading a file as lines
    """
    included_files = ['/etc/cups/cupsd.conf']
    error_list = []

    vetted_included_files = []
    while included_files:
        # NOTE(ivasilev) Will be using stack to process last encountered include directives first
        included_file = included_files.pop(-1)
        try:
            lines = read_func(included_file)
        except IOError:
            error_list.append('Error during reading file {}: file not'
                              ' found'.format(included_file))
            continue
        # Append to the resulting list of vetted files if exception wasn't raised
        vetted_included_files.append(included_file)
        # Mark any other included file you find as need-to-be-validated
        includes_to_process = []
        for line in lines:
            value = get_directive_value('Include', line)
            if value:
                includes_to_process.append(value)
        # NOTE(ivasilev) Add discovered Include directives to the stack in reversed order, so that they are processed
        # in the same order they appeared in the file
        included_files.extend(reversed(includes_to_process))

    return (vetted_included_files, error_list)


def digest_directive_check(path, read_func=_read_file):
    """
    Checks if AuthType or DefaultAuthType directives contain
    Digest or BasicDigest values, which were removed.

    :param str path: path to configuration file
    :param func read_func: function for reading the file
    """
    lines = read_func(path)

    for line in lines:
        for name in ['AuthType', 'DefaultAuthType']:
            for value in ['Digest', 'BasicDigest']:
                found_value = get_directive_value(name, line)
                if found_value == value:
                    return True
    return False


def ssl_directive_check(read_func=_read_file):
    """
    Checks if ServerCertificate or ServerKey directives are
    used in cups-files.conf.

    :param func read_func: function for reading the file
    """
    lines = read_func('/etc/cups/cups-files.conf')

    for line in lines:
        for name in ['ServerCertificate', 'ServerKey']:
            value = get_directive_value(name, line)
            if value:
                return True
    return False


def environment_setup_check(path, read_func=_read_file):
    """
    Checks if PassEnv or SetEnv directives are used in configuration.
    They were moved to cups-files.conf in newer CUPS due security
    issues.

    :param str path: path to configuration file
    :param func read_func: reads the file
    """
    lines = read_func(path)

    for line in lines:
        for name in ['SetEnv', 'PassEnv']:
            value = get_directive_value(name, line)
            if value:
                return True
    return False


def print_capabilities_check(path, read_func=_read_file):
    """
    Checks if PrintcapFormat directive is used in configuration.
    It was moved to cups-files.conf in newer CUPS.

    :param str path: path to configuration file
    :param func read_func: reads the file
    """
    lines = read_func(path)

    for line in lines:
        value = get_directive_value('PrintcapFormat', line)
        if value:
            return True
    return False


def _send_model(interface, digest, include, certkey, env,
                printcap, include_files_list):
    """
    Produces model of facts.

    :param bool interface: true if interface scripts are used
    :param bool digest: true if BasicDigest/Digest values are used
    :param bool include: true if Include directive is used
    :param bool certkey: true if ServerCertificate/ServerKey directives are used
    :param bool env: true if PassEnv/SetEnv directives are used
    :param bool printcap: true if PrintcapFormat directive is used
    :param list include_files_list: contains paths to included files
    """
    api.produce(CupsChangedFeatures(interface=interface,
                                    digest=digest,
                                    include=include,
                                    certkey=certkey,
                                    env=env,
                                    printcap=printcap,
                                    include_files=include_files_list))


def find_features(debug_log=api.current_logger().debug,
                  warn_log=api.current_logger().warn,
                  error_log=api.current_logger().error,
                  send_features=_send_model,
                  is_installed=_check_package,
                  read_func=_read_file,
                  check_path_func=_path_exists,
                  list_dir_func=_list_dir):
    """
    Checks every feature which changed between CUPS 1.6.3 and CUPS
    2.2.6.

    :param func debug_log: function for debug logging
    :param func error_log: function for error logging
    :param func warn_log: function for warning logging
    :param func send_features: produces CupsMigrationModel if necessary
    :param func is_installed: check if the package is installed
    :param func read_func: reads a file
    :param func check_path_func: checks if the file exists
    :param func list_dir_func: list files in a directory
    """

    if not is_installed('cups'):
        return

    if (
        not check_path_func('/etc/cups/cupsd.conf') or
        not check_path_func('/etc/cups/cups-files.conf')
    ):
        # seatbelt - it's expected as super rare to have malfunction cupsd :)
        raise StopActorExecutionError('Core CUPS configuration files '
                                      'are missing. CUPS installation '
                                      'is corrupted, terminating the actor.')

    debug_log('Checking if CUPS configuration contains removed features.')

    digest = env = printcap = interface = certkey = include = False

    include_file_list, error_list = include_directive_check(read_func)

    if error_list:
        warn_log('Following included files will not be appended to '
                 'cupsd.conf due attached error:'
                 + ''.join(['\n   - {}'.format(err) for err in error_list]))

    if len(include_file_list) > 1:
        include = True

    interface = interface_script_check(check_path_func, list_dir_func)

    for config_file in include_file_list:

        if not digest:
            digest = digest_directive_check(config_file, read_func)

        if not env:
            env = environment_setup_check(config_file, read_func)

        if not printcap:
            printcap = print_capabilities_check(config_file, read_func)

    certkey = ssl_directive_check(read_func)

    if any([interface, digest, include, certkey, env, printcap]):
        send_features(interface, digest, include, certkey, env,
                      printcap, include_file_list)



from leapp.actors import Actor
from leapp.libraries.actor.satellite_upgrade_check import satellite_upgrade_check
from leapp.models import Report, SatelliteFacts
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class SatelliteUpgradeCheck(Actor):
    """
    Check state of Satellite system before upgrade
    """

    name = 'satellite_upgrade_check'
    consumes = (SatelliteFacts,)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        facts = next(self.consume(SatelliteFacts), None)
        if not facts or not facts.has_foreman:
            return

        satellite_upgrade_check(facts)



from leapp.actors import Actor
from leapp.libraries.actor import insightsautoregister
from leapp.models import InstalledRPM
from leapp.reporting import Report
from leapp.tags import FirstBootPhaseTag, IPUWorkflowTag


class InsightsAutoregister(Actor):
    """
    Automatically registers system into Red Hat Insights

    The registration is skipped if NO_INSIGHTS_REGISTER=1 environment variable
    is set, the --no-insights-register command line argument present or the
    system isn't registered with subscription-manager.
    """

    name = 'insights_auto_register'
    consumes = (InstalledRPM,)
    produces = (Report,)
    tags = (FirstBootPhaseTag, IPUWorkflowTag)

    def process(self):
        insightsautoregister.process()



import os

from leapp import reporting
from leapp.libraries.stdlib import api
from leapp.models import StorageInfo

FMT_LIST_SEPARATOR = '\n    - '


def _get_common_path(path1, path2):
    """
    Return the longest common absolute sub-path for pair of given absolute paths.

    Note that this function implements similar functionality as os.path.commonpath(), however this function is not
    available in python2.7, thus can't be used here.
    """

    if not path1 or not path2:
        return ''

    path1 = path1.strip('/').split('/')
    path2 = path2.strip('/').split('/')

    common_path = []
    for path1_part, path2_part in zip(path1, path2):
        if path1_part != path2_part:
            break
        common_path.append(path1_part)
    return os.path.join('/', *common_path)


def _get_overshadowing_mount_points(mount_points):
    """
    Retrieve set of overshadowing and overshadowed mount points.

    :param list[str] mount_points: absolute paths to mount points without trailing /
    :returns: set of unique mount points without trailing /
    """
    overshadowing = set()
    for i, mount_point in enumerate(mount_points):
        for overshadowing_mount_point in mount_points[i+1:]:
            if _get_common_path(mount_point, overshadowing_mount_point) == overshadowing_mount_point:
                overshadowing.add(overshadowing_mount_point)
                overshadowing.add(mount_point)
    return overshadowing


def check_fstab_mount_order():
    storage_info = next(api.consume(StorageInfo), None)

    if not storage_info:
        return

    mount_points = []
    for fstab_entry in storage_info.fstab:
        mount_point = fstab_entry.fs_file
        if mount_point != '/':
            mount_point = mount_point.rstrip('/')
        if os.path.isabs(mount_point):
            mount_points.append(mount_point)

    overshadowing = _get_overshadowing_mount_points(mount_points)
    duplicates = {mp for mp in mount_points if mount_points.count(mp) > 1}

    if not overshadowing:
        return

    overshadowing_in_order = [mp for mp in mount_points if mp in overshadowing]
    overshadowing_fixed = sorted(set(mount_points), key=len)
    summary = 'Leapp detected incorrect /etc/fstab format that causes overshadowing of mount points.'
    hint = 'To prevent the overshadowing:'

    if duplicates:
        summary += '\nDetected mount points with duplicates: {}'.format(', '.join(duplicates))
        hint += ' Remove detected duplicates by using unique mount points.'

    if overshadowing:
        summary += '\nDetected order of overshadowing mount points: {}'.format(', '.join(overshadowing_in_order))
        hint += (
            ' Reorder the detected overshadowing entries. Possible order of all mount '
            'points without overshadowing:{}{}'
        ).format(FMT_LIST_SEPARATOR, FMT_LIST_SEPARATOR.join(overshadowing_fixed))

    reporting.create_report([
        reporting.Title(
            'Detected incorrect order of entries or duplicate entries in /etc/fstab, preventing a successful '
            'in-place upgrade.'
        ),
        reporting.Summary(summary),
        reporting.Remediation(hint=hint),
        reporting.RelatedResource('file', '/etc/fstab'),
        reporting.Severity(reporting.Severity.HIGH),
        reporting.Groups([reporting.Groups.FILESYSTEM]),
        reporting.Groups([reporting.Groups.INHIBITOR]),
    ])



import os

import pytest

from leapp.libraries.actor.pcidevicesscanner import parse_pci_devices, produce_pci_devices
from leapp.models import PCIDevice, PCIDevices


def test_parse_pci_devices():
    devices_textual = '''Slot:	00:00.0
Class:	Host bridge
Vendor:	Intel Corporation
Device:	440FX - 82441FX PMC [Natoma]
SVendor:	Red Hat, Inc.
SDevice:	Qemu virtual machine
PhySlot:	3
Rev:	02
NUMANode:	0

Slot:	00:01.0
Class:	ISA bridge
Vendor:	Intel Corporation
Device:	82371SB PIIX3 ISA [Natoma/Triton II]
SVendor:	Red Hat, Inc.
SDevice:	Qemu virtual machine

Slot:	00:01.1
Class:	IDE interface
Vendor:	Intel Corporation
Device:	82371SB PIIX3 IDE [Natoma/Triton II]
ProgIf:	80
Driver:	ata_piix
Module:	ata_piix
Module:	pata_acpi
Module:	ata_generic

'''
    devices_numeric = '''Slot:	00:00.0
Class:	Host bridge
Vendor:	15b45
Device:	0724
SVendor:	15b46
SDevice:	0725
PhySlot:	3
Rev:	02
NUMANode:	0

Slot:	00:01.0
Class:	ISA bridge
Vendor:	15b44
Device:	0723
SVendor:	15b50
SDevice:	0750

Slot:	00:01.1
Class:	IDE interface
Vendor:	15b43
Device:	0722
ProgIf:	80
Driver:	ata_piix
Module:	ata_piix
Module:	pata_acpi
Module:	ata_generic

'''

    output = parse_pci_devices(devices_textual, devices_numeric)
    assert isinstance(output, list)
    assert len(output) == 3

    dev = output.pop()
    assert dev.slot == '00:01.1'
    assert dev.dev_cls == 'IDE interface'
    assert dev.vendor == 'Intel Corporation'
    assert dev.name == '82371SB PIIX3 IDE [Natoma/Triton II]'
    assert dev.progif == '80'
    assert dev.driver == 'ata_piix'
    assert dev.pci_id == '15b43:0722'
    assert len(dev.modules) == 3
    assert 'ata_piix' in dev.modules
    assert 'pata_acpi' in dev.modules
    assert 'ata_generic' in dev.modules

    dev = output.pop()
    assert dev.slot == '00:01.0'
    assert dev.dev_cls == 'ISA bridge'
    assert dev.vendor == 'Intel Corporation'
    assert dev.name == '82371SB PIIX3 ISA [Natoma/Triton II]'
    assert dev.subsystem_vendor == 'Red Hat, Inc.'
    assert dev.subsystem_name == 'Qemu virtual machine'
    assert dev.driver == ''
    assert dev.modules == []
    assert dev.rev == ''
    assert dev.physical_slot == ''
    assert dev.numa_node == ''
    assert dev.pci_id == '15b44:0723:15b50:0750'

    dev = output.pop()
    assert dev.slot == '00:00.0'
    assert dev.dev_cls == 'Host bridge'
    assert dev.vendor == 'Intel Corporation'
    assert dev.name == '440FX - 82441FX PMC [Natoma]'
    assert dev.subsystem_vendor == 'Red Hat, Inc.'
    assert dev.subsystem_name == 'Qemu virtual machine'
    assert dev.rev == '02'
    assert dev.physical_slot == '3'
    assert dev.numa_node == '0'
    assert dev.pci_id == '15b45:0724:15b46:0725'


def test_parse_empty_list():
    output = parse_pci_devices('', '')
    assert isinstance(output, list)
    assert not output


def test_parse_unknown_keys():
    devices_textual = '''Slot:	00:1c.0
Class:	PCI bridge
Material:	Silicon
Vendor:	Intel Corporation
Origin:	People's Republic of China
Device:	Sunrise Point-LP PCI Express Root Port #1
Flavor:	Burnt toast
Rev:	f1
Flavor:	Spicy beef
Driver:	pcieport

'''
    devices_numeric = '''Slot:	00:1c.0
Class:	PCI bridge
Material:	Silicon
Vendor:	15b74
Origin:	People's Republic of China
Device:	0724
Flavor:	Burnt toast
Rev:	f1
Flavor:	Spicy beef
Driver:	pcieport

'''

    output = parse_pci_devices(devices_textual, devices_numeric)
    assert isinstance(output, list)
    assert len(output) == 1

    dev = output.pop()
    assert dev.slot == '00:1c.0'
    assert dev.dev_cls == 'PCI bridge'
    assert dev.vendor == 'Intel Corporation'
    assert dev.name == 'Sunrise Point-LP PCI Express Root Port #1'
    assert dev.rev == 'f1'
    assert dev.driver == 'pcieport'
    assert dev.pci_id == '15b74:0724'
    assert dev.modules == []


def test_produce_pci_devices():
    output = []

    def fake_producer(*args):
        output.extend(args)

    devices = [
        PCIDevice(
            slot='00:00.0',
            dev_cls='Host bridge',
            vendor='Intel Corporation',
            name='440FX - 82441FX PMC [Natoma]',
            subsystem_vendor='Red Hat, Inc.',
            subsystem_name='Qemu virtual machine',
            pci_id='15b560:0739',
            rev='02'),
        PCIDevice(
            slot='00:01.0',
            dev_cls='ISA bridge',
            vendor='Intel Corporation',
            name='82371SB PIIX3 ISA [Natoma/Triton II]',
            subsystem_vendor='Red Hat, Inc.',
            pci_id='15b560:0739',
            subsystem_name='Qemu virtual machine'),
        PCIDevice(
            slot='00:01.1',
            dev_cls='IDE interface',
            vendor='Intel Corporation',
            name='82371SB PIIX3 IDE [Natoma/Triton II]',
            subsystem_vendor='Red Hat, Inc.',
            subsystem_name='Qemu virtual machine',
            pci_id='15b560:0739',
            progif='80'),
    ]

    produce_pci_devices(fake_producer, devices)
    assert len(output) == 1
    assert len(output[0].devices) == 3


def test_produce_no_devices():
    output = []

    def fake_producer(*args):
        output.extend(args)

    produce_pci_devices(fake_producer, [])
    assert len(output) == 1
    assert not output[0].devices


# TODO(pstodulk): update the test - drop current_actor_context and use monkeypatch
@pytest.mark.skipif(not os.path.exists('/usr/sbin/lspci'), reason='lspci not installed on the system')
def test_actor_execution(current_actor_context):
    current_actor_context.run()
    assert current_actor_context.consume(PCIDevices)



from leapp.libraries.common import config
from leapp.models import FstabEntry, StorageInfo
from leapp.reporting import Report
from leapp.snactor.fixture import current_actor_context
from leapp.utils.report import is_inhibitor


def test_actor_with_fstab_entry(current_actor_context, monkeypatch):
    monkeypatch.setattr(config, 'get_env', lambda x, y: y)
    with_fstab_entry = [FstabEntry(fs_spec="//10.20.30.42/share1", fs_file="/mnt/win_share1",
                                   fs_vfstype="cifs",
                                   fs_mntops="credentials=/etc/win-credentials,file_mode=0755,dir_mode=0755",
                                   fs_freq="0", fs_passno="0"),
                        FstabEntry(fs_spec="//10.20.30.42/share2", fs_file="/mnt/win_share2",
                                   fs_vfstype="cifs",
                                   fs_mntops="credentials=/etc/win-credentials,file_mode=0755,dir_mode=0755",
                                   fs_freq="0", fs_passno="0"),
                        FstabEntry(fs_spec="/dev/mapper/fedora-home", fs_file="/home",
                                   fs_vfstype="ext4",
                                   fs_mntops="defaults,x-systemd.device-timeout=0",
                                   fs_freq="1", fs_passno="2")]
    current_actor_context.feed(StorageInfo(fstab=with_fstab_entry))
    current_actor_context.run()
    report_fields = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fields)
    assert report_fields['severity'] == 'high'
    assert report_fields['title'] == "Use of CIFS detected. Upgrade can't proceed"


def test_actor_no_cifs(current_actor_context, monkeypatch):
    monkeypatch.setattr(config, 'get_env', lambda x, y: y)
    with_fstab_entry = [FstabEntry(fs_spec="/dev/mapper/fedora-home", fs_file="/home",
                                   fs_vfstype="ext4",
                                   fs_mntops="defaults,x-systemd.device-timeout=0",
                                   fs_freq="1", fs_passno="2")]
    current_actor_context.feed(StorageInfo(fstab=with_fstab_entry))
    current_actor_context.run()
    assert not current_actor_context.consume(Report)



import xml.etree.ElementTree as ElementTree

from leapp.actors import Actor
from leapp.libraries.actor import firewalldupdatelockdownwhitelist
from leapp.models import FirewalldFacts
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag


class FirewalldUpdateLockdownWhitelist(Actor):
    """
    Update the firewalld Lockdown Whitelist.

    RHEL-8 uses a platform specific python interpreter for packaged
    applications. For firewall-config, the interpreter path is part of the
    lockdown list. In RHEL-7 this was simply /usr/bin/python, but in RHEL-8
    it's /usr/libexec/platform-python. However, if the user made changes to the
    lockdown whitelist it won't be replaced by RPM/dnf. As such we must update
    the interpreter if the old value is there.
    """

    name = 'firewalld_update_lockdown_whitelist'
    consumes = (FirewalldFacts,)
    produces = ()
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)

    def process(self):
        for facts in self.consume(FirewalldFacts):
            if facts.firewall_config_command:
                tree = ElementTree.parse('/etc/firewalld/lockdown-whitelist.xml')
                root = tree.getroot()

                need_write = firewalldupdatelockdownwhitelist.updateFirewallConfigCommand(
                    root,
                    facts.firewall_config_command
                )

                if need_write:
                    tree.write('/etc/firewalld/lockdown-whitelist.xml')
                    self.log.info('Updated lockdown whitelist')



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic
from leapp.utils.deprecation import deprecated


class RPM(Model):
    topic = SystemInfoTopic
    name = fields.String()
    epoch = fields.String()
    packager = fields.String()
    version = fields.String()
    release = fields.String()
    arch = fields.String()
    pgpsig = fields.String()
    repository = fields.Nullable(fields.String())
    module = fields.Nullable(fields.String())
    stream = fields.Nullable(fields.String())


class InstalledRPM(Model):
    topic = SystemInfoTopic
    items = fields.List(fields.Model(RPM), default=[])


class DistributionSignedRPM(InstalledRPM):
    pass


@deprecated(since='2024-01-31', message='Replaced by DistributionSignedRPM')
class InstalledRedHatSignedRPM(InstalledRPM):
    pass


class InstalledUnsignedRPM(InstalledRPM):
    pass



import os

from leapp.libraries.stdlib import api
from leapp.models import CopyFile, TargetUserSpacePreupgradeTasks

# Maps src location in the source system to the destination within the target system
FILES_TO_COPY_IF_PRESENT = {
    '/etc/hosts': '/etc/hosts'
}


def scan_files_to_copy():
    """
    Scans the source system and identifies files that should be copied into target userspace.

    When an item to be copied is identified a message :class:`CopyFile` is produced.
    """
    files_to_copy = []
    for src_path in FILES_TO_COPY_IF_PRESENT:
        if os.path.isfile(src_path):
            dst_path = FILES_TO_COPY_IF_PRESENT[src_path]
            files_to_copy.append(CopyFile(src=src_path, dst=dst_path))

    preupgrade_task = TargetUserSpacePreupgradeTasks(copy_files=files_to_copy)

    api.produce(preupgrade_task)



from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.common.rpms import has_package
from leapp.models import InstalledRPM
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckDocker(Actor):
    """
    Checks if Docker is installed and warns about its deprecation in RHEL8.
    """

    name = 'check_docker'
    consumes = (InstalledRPM,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        if has_package(InstalledRPM, 'docker'):
            create_report([
                reporting.Title('Transition from Docker to Podman in RHEL8'),
                reporting.Summary('Docker has been deprecated in favour of Podman in Red Hat Enterprise Linux 8. The '
                                  'docker package is going to be removed during the upgrade without migration of '
                                  'existing containers.'),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups([reporting.Groups.TOOLS]),
                reporting.Remediation(hint='It is recommended to re-create the containers with the appropriate '
                                           'container images and reattach any in-use volumes using podman directly '
                                           'prior to the upgrade of the operating system, which should provide the '
                                           'same level of functionality. '),
                reporting.RelatedResource('package', 'docker'),
                reporting.ExternalLink(url='https://access.redhat.com/solutions/5213331',
                                       title='How do I migrate my Docker containers to Podman prior to moving from '
                                             'Red Hat Enterprise Linux 7 to Red Hat Enterprise Linux 8?')
            ])



import os

from leapp import reporting
from leapp.libraries.actor import customnetworkscripts
from leapp.libraries.common import testutils


def test_customnetworkscripts_exists(monkeypatch):
    monkeypatch.setattr(os.path, "isfile", lambda dummy: True)
    monkeypatch.setattr(reporting, "create_report", testutils.create_report_mocked())
    customnetworkscripts.process()
    assert reporting.create_report.called


def test_customnetworkscripts_not_found(monkeypatch):
    monkeypatch.setattr(os.path, "isfile", lambda dummy: False)
    monkeypatch.setattr(reporting, 'create_report', testutils.create_report_mocked())
    customnetworkscripts.process()
    assert not reporting.create_report.called



import xml.etree.ElementTree as ElementTree

from leapp.libraries.actor import firewalldfactsactor


def test_firewalldfactsactor_direct():
    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <direct>
             <passthrough ipv="eb">-t broute -I BROUTING 1 -j ACCEPT</passthrough>
           </direct>
        ''')
    assert firewalldfactsactor.getEbtablesTablesInUse(root) == ['broute']

    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <direct>
             <rule priority="1" table="broute" ipv="eb" chain="BROUTING">-j ACCEPT</rule>
           </direct>
        ''')
    assert firewalldfactsactor.getEbtablesTablesInUse(root) == ['broute']

    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <direct>
             <rule priority="1" table="broute" ipv="eb" chain="BROUTING">-j ACCEPT</rule>
             <rule priority="1" table="filter" ipv="ipv4" chain="INPUT">-j ACCEPT</rule>
             <passthrough ipv="eb">-t nat -I PREROUTING 1 -j ACCEPT</passthrough>
           </direct>
        '''
    )
    assert set(firewalldfactsactor.getEbtablesTablesInUse(root)) == set(['broute', 'nat'])


def test_firewalldfactsactor_firewallConfigCommand():
    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <whitelist>
             <command name="/usr/bin/python -Es /usr/bin/firewall-config"/>
             <command name="/usr/bin/foobar"/>
             <selinux context="system_u:system_r:NetworkManager_t:s0"/>
             <selinux context="system_u:system_r:virtd_t:s0-s0:c0.c1023"/>
             <user id="0"/>
           </whitelist>
        '''
    )
    assert firewalldfactsactor.getLockdownFirewallConfigCommand(root) == '/usr/bin/python -Es /usr/bin/firewall-config'

    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <whitelist>
             <command name="/usr/bin/foobar"/>
           </whitelist>
        ''')
    assert firewalldfactsactor.getLockdownFirewallConfigCommand(root) == ''

    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <whitelist>
             <command name="/usr/libexec/platform-python -s /usr/bin/firewall-config"/>
             <selinux context="system_u:system_r:NetworkManager_t:s0"/>
             <selinux context="system_u:system_r:virtd_t:s0-s0:c0.c1023"/>
             <user id="0"/>
           </whitelist>
        ''')
    EXP_RESULT = '/usr/libexec/platform-python -s /usr/bin/firewall-config'
    assert firewalldfactsactor.getLockdownFirewallConfigCommand(root) == EXP_RESULT


def test_firewalldfactsactor_ipsetTypes():
    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <ipset type="hash:ip">
             <short>My Ipset</short>
             <description>description</description>
             <entry>1.2.3.4</entry>
           </ipset>
        ''')
    assert firewalldfactsactor.getIpsetTypesInUse(root) == ['hash:ip']

    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <ipset type="hash:net,port">
             <short>My Ipset</short>
             <description>description</description>
           </ipset>
        ''')
    assert firewalldfactsactor.getIpsetTypesInUse(root) == ['hash:net,port']



from leapp.libraries.common import rhsm
from leapp.libraries.common.mounting import NotIsolatedActions
from leapp.libraries.stdlib import api


@rhsm.with_rhsm
def scan():
    context = NotIsolatedActions(base_dir='/')
    info = rhsm.scan_rhsm_info(context)
    api.produce(info)



from leapp.actors import Actor
from leapp.libraries.actor.repositoriesmapping import scan_repositories
from leapp.models import ConsumedDataAsset, RepositoriesMapping
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class RepositoriesMappingScanner(Actor):
    """
    Produces message containing repository mapping based on provided file.

    The actor filters out data irrelevant to the current IPU (data with different
    source/target major versions) from the raw repository mapping data.
    """

    name = 'repository_mapping'
    consumes = ()
    produces = (ConsumedDataAsset, RepositoriesMapping,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        scan_repositories()



from leapp.models import IfCfg, IfCfgProperty, InstalledRPM, RPM, RpmTransactionTasks
from leapp.reporting import Report
from leapp.utils.report import is_inhibitor

RH_PACKAGER = "Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>"

NETWORK_SCRIPTS_RPM = RPM(
    name="network-scripts",
    version="10.00.17",
    release="1.el8",
    epoch="",
    packager=RH_PACKAGER,
    arch="x86_64",
    pgpsig="RSA/SHA256, Fri 04 Feb 2022 03:32:47 PM CET, Key ID 199e2f91fd431d51",
)

NETWORK_MANAGER_RPM = RPM(
    name="NetworkManager",
    version="1.36.0",
    release="0.8.el8",
    epoch="1",
    packager=RH_PACKAGER,
    arch="x86_64",
    pgpsig="RSA/SHA256, Mon 14 Feb 2022 08:45:37 PM CET, Key ID 199e2f91fd431d51",
)

INITSCRIPTS_INSTALLED = InstalledRPM(items=[
    NETWORK_SCRIPTS_RPM
])
INITSCRIPTS_AND_NM_INSTALLED = InstalledRPM(items=[
    NETWORK_SCRIPTS_RPM,
    NETWORK_MANAGER_RPM
])


def test_ifcfg_none(current_actor_context):
    """
    No report and don't install anything if there are no ifcfg files.
    """

    current_actor_context.feed(INITSCRIPTS_AND_NM_INSTALLED)
    current_actor_context.run()
    assert not current_actor_context.consume(Report)
    assert not current_actor_context.consume(RpmTransactionTasks)


def test_ifcfg_rule_file(current_actor_context):
    """
    Install NetworkManager-dispatcher-routing-rules package if there's a
    file with ip rules.
    """

    current_actor_context.feed(IfCfg(
        filename="/NM/ifcfg-eth0",
        properties=(IfCfgProperty(name="TYPE", value="Ethernet"),),
        rules=("foo bar baz",),
    ))
    current_actor_context.feed(INITSCRIPTS_AND_NM_INSTALLED)
    current_actor_context.run()
    assert not current_actor_context.consume(Report)
    assert len(current_actor_context.consume(RpmTransactionTasks)) == 1
    rpm_transaction = current_actor_context.consume(RpmTransactionTasks)[0]
    assert rpm_transaction.to_install == ["NetworkManager-dispatcher-routing-rules"]


def test_ifcfg_good_type(current_actor_context):
    """
    No report if there's an ifcfg file that would work with NetworkManager.
    Make sure NetworkManager itself is installed though.
    """

    current_actor_context.feed(IfCfg(
        filename="/NM/ifcfg-lo",
        properties=()
    ))
    current_actor_context.feed(IfCfg(
        filename="/NM/ifcfg-eth0",
        properties=(IfCfgProperty(name="TYPE", value="Ethernet"),)
    ))
    current_actor_context.feed(INITSCRIPTS_AND_NM_INSTALLED)
    current_actor_context.run()
    assert not current_actor_context.consume(Report)
    assert len(current_actor_context.consume(RpmTransactionTasks)) == 1
    rpm_transaction = current_actor_context.consume(RpmTransactionTasks)[0]
    assert rpm_transaction.to_install == ["NetworkManager"]


def test_ifcfg_not_controlled(current_actor_context):
    """
    Report if there's a NM_CONTROLLED=no file.
    """

    current_actor_context.feed(IfCfg(
        filename="/NM/ifcfg-eth0",
        properties=(
            IfCfgProperty(name="TYPE", value="Ethernet"),
            IfCfgProperty(name="NM_CONTROLLED", value="no"),
        )
    ))
    current_actor_context.feed(INITSCRIPTS_INSTALLED)
    current_actor_context.run()
    assert len(current_actor_context.consume(Report)) == 1
    report_fields = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fields)
    assert "disabled NetworkManager" in report_fields['title']


def test_ifcfg_unknown_type(current_actor_context):
    """
    Report if there's configuration for a type we don't recognize.
    """

    current_actor_context.feed(IfCfg(
        filename="/NM/ifcfg-pigeon0",
        properties=(IfCfgProperty(name="TYPE", value="AvianCarrier"),)
    ))
    current_actor_context.feed(INITSCRIPTS_AND_NM_INSTALLED)
    current_actor_context.run()
    assert len(current_actor_context.consume(Report)) == 1
    report_fields = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fields)
    assert "unsupported device types" in report_fields['title']


def test_ifcfg_install_subpackage(current_actor_context):
    """
    Install NetworkManager-team if there's a team connection and also
    ensure NetworkManager-config-server is installed if NetworkManager
    was not there.
    """

    current_actor_context.feed(IfCfg(
        filename="/NM/ifcfg-team0",
        properties=(IfCfgProperty(name="TYPE", value="Team"),)
    ))
    current_actor_context.feed(INITSCRIPTS_INSTALLED)
    current_actor_context.run()
    assert not current_actor_context.consume(Report)
    assert len(current_actor_context.consume(RpmTransactionTasks)) == 1
    rpm_transaction = current_actor_context.consume(RpmTransactionTasks)[0]
    assert rpm_transaction.to_install == [
        "NetworkManager-team",
        "NetworkManager-config-server",
    ]



import pyudev

from leapp.libraries.stdlib import api
from leapp.models import Interface, PCIAddress

udev_context = pyudev.Context()


def physical_interfaces():
    """
    Returns a list of pyudev.Device objects for all physical network interfaces
    """
    enumerator = pyudev.Enumerator(udev_context).match_subsystem('net')
    return [d for d in enumerator if not d.device_path.startswith('/devices/virtual/')]


def pci_info(path):
    """
    Returns PCI topology info from string which is expected to be a value of ID_PATH udev device property
    """
    pci = {}

    # TODO(msekleta): check that path argument actually has ID_PATH format
    if path.startswith('pci-'):
        components = path[4:16].split(':')
        pci['domain'] = components[0]
        pci['bus'] = components[1]
        pci['device'] = components[2].split('.')[0]
        pci['function'] = components[2].split('.')[1]

    return pci


def interfaces():
    """
    Generator which produces an Interface objects containing assorted interface properties relevant for network naming
    """
    for dev in physical_interfaces():
        attrs = {}

        try:
            attrs['name'] = dev.sys_name
            attrs['devpath'] = dev.device_path
            attrs['driver'] = dev['ID_NET_DRIVER']
            attrs['vendor'] = dev['ID_VENDOR_ID']
            attrs['pci_info'] = PCIAddress(**pci_info(dev['ID_PATH']))
            attrs['mac'] = dev.attributes.get('address')
            if isinstance(attrs['mac'], bytes):
                attrs['mac'] = attrs['mac'].decode()
        except Exception as e:  # pylint: disable=broad-except
            # FIXME(msekleta): We should probably handle errors more granularly
            # Maybe we should inhibit upgrade process at this point
            api.current_logger().warning('Failed to gather information about network interface: ' + str(e))
            continue

        yield Interface(**attrs)



from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.common.config import get_env
from leapp.models import StorageInfo
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckNfs(Actor):
    """
    Check if NFS filesystem is in use. If yes, inhibit the upgrade process.

    Actor looks for NFS in the following sources: /ets/fstab and mount.
    If there is NFS in any of the mentioned sources, actors inhibits the upgrade.
    """
    name = "check_nfs"
    consumes = (StorageInfo,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag,)

    def process(self):
        # if network in initramfs is enabled NFS inhibitors are redundant
        if get_env('LEAPP_DEVEL_INITRAM_NETWORK', None):
            return
        details = "NFS is currently not supported by the inplace upgrade.\n" \
                  "We have found NFS usage at the following locations:\n"

        def _is_nfs(a_type):
            return a_type.startswith('nfs') and a_type != 'nfsd'

        for storage in self.consume(StorageInfo):
            # Check fstab
            fstab_nfs_mounts = []
            for fstab in storage.fstab:
                if _is_nfs(fstab.fs_vfstype):
                    fstab_nfs_mounts.append(" - {} {}\n".format(fstab.fs_spec, fstab.fs_file))

            # Check mount
            nfs_mounts = []
            for mount in storage.mount:
                if _is_nfs(mount.tp):
                    nfs_mounts.append(" - {} {}\n".format(mount.name, mount.mount))

        if any((fstab_nfs_mounts, nfs_mounts)):
            if fstab_nfs_mounts:
                details += "- NFS shares found in /etc/fstab:\n"
                details += ''.join(fstab_nfs_mounts)

            if nfs_mounts:
                details += "- NFS shares currently mounted:\n"
                details += ''.join(nfs_mounts)

            fstab_related_resource = [reporting.RelatedResource('file', '/etc/fstab')] if fstab_nfs_mounts else []

            create_report([
                reporting.Title("Use of NFS detected. Upgrade can't proceed"),
                reporting.Summary(details),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups([
                        reporting.Groups.FILESYSTEM,
                        reporting.Groups.NETWORK
                ]),
                reporting.Remediation(hint='Disable NFS temporarily for the upgrade if possible.'),
                reporting.Groups([reporting.Groups.INHIBITOR]),
                ] + fstab_related_resource
            )



import functools

from leapp import reporting
from leapp.libraries.actor import checkvdo
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import (
    VdoConversionInfo,
    VdoConversionPostDevice,
    VdoConversionPreDevice,
    VdoConversionUndeterminedDevice
)
from leapp.utils.report import is_inhibitor


# Mock actor base for CheckVdo tests.
class MockedActorCheckVdo(CurrentActorMocked):
    def get_vdo_answer(self):
        return False


# Mock actor for all_vdo_converted dialog response.
class MockedActorAllVdoConvertedTrue(MockedActorCheckVdo):
    def get_vdo_answer(self):
        return True


def aslist(f):
    """ Decorator used to convert generator to list """
    @functools.wraps(f)
    def inner(*args, **kwargs):
        return list(f(*args, **kwargs))
    return inner


@aslist
def _post_conversion_vdos(count=0, complete=0, failing=0, start_char='a'):
    begin = complete
    for x in range(begin):
        yield VdoConversionPostDevice(name='sd{0}'.format(chr(ord(start_char) + x)),
                                      complete=True)

    for x in range(begin, begin + failing):
        yield VdoConversionPostDevice(name='sd{0}'.format(chr(ord(start_char) + x)),
                                      complete=False,
                                      check_failed=True,
                                      failure='unit testing')
    begin += failing

    for x in range(begin, count):
        yield VdoConversionPostDevice(name='sd{0}'.format(chr(ord(start_char) + x)),
                                      complete=False)


@aslist
def _pre_conversion_vdos(count=0, start_char='a'):
    for x in range(count):
        yield VdoConversionPreDevice(name='sd{0}'.format(chr(ord(start_char) + x)))


@aslist
def _undetermined_conversion_vdos(count=0, failing=False, start_char='a'):
    for x in range(count):
        yield VdoConversionUndeterminedDevice(name='sd{0}'.format(chr(ord(start_char) + x)),
                                              check_failed=failing,
                                              failure='unit testing' if failing else None)


# No VDOs tests.
def test_no_vdos(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos()))
    assert reporting.create_report.called == 0


# Concurrent pre- and post-conversion tests.
def test_both_conversion_vdo_incomplete(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    post_count = 7
    checkvdo.check_vdo(
        VdoConversionInfo(
            post_conversion=_post_conversion_vdos(post_count, 5),
            pre_conversion=_pre_conversion_vdos(3, start_char=chr(ord('a') + post_count)),
            undetermined_conversion=_undetermined_conversion_vdos()))
    assert reporting.create_report.called == 2
    assert is_inhibitor(reporting.create_report.report_fields)


# Post-conversion tests.
def test_post_conversion_multiple_vdo_incomplete(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(7, 5),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos()))
    assert reporting.create_report.called == 1
    assert is_inhibitor(reporting.create_report.report_fields)
    assert reporting.create_report.report_fields['summary'].startswith('VDO devices')


def test_post_conversion_multiple_vdo_complete(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(7, 7),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos()))
    assert reporting.create_report.called == 0


def test_post_conversion_single_vdo_incomplete(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(1),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos()))
    assert reporting.create_report.called == 1
    assert is_inhibitor(reporting.create_report.report_fields)
    assert (reporting.create_report.report_fields['summary'].startswith('VDO device')
            and (not reporting.create_report.report_fields['summary'].startswith('VDO devices')))


def test_post_conversion_single_check_failing(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(2, complete=1, failing=1),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos()))
    assert reporting.create_report.called == 1
    assert is_inhibitor(reporting.create_report.report_fields)
    assert (reporting.create_report.report_fields['summary'].startswith(
            'Unexpected result checking device') and
            (not reporting.create_report.report_fields['summary'].startswith(
             'Unexpected result checking devices')))


def test_post_conversion_multiple_check_failing(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(7, complete=4, failing=3),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos()))
    assert reporting.create_report.called == 1
    assert is_inhibitor(reporting.create_report.report_fields)
    assert reporting.create_report.report_fields['summary'].startswith(
            'Unexpected result checking devices')


def test_post_conversion_incomplete_and_check_failing(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(2, failing=1),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos()))
    assert reporting.create_report.called == 2
    assert is_inhibitor(reporting.create_report.report_fields)


# Pre-conversion tests.
def test_pre_conversion_multiple_vdo_incomplete(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(),
                          pre_conversion=_pre_conversion_vdos(7),
                          undetermined_conversion=_undetermined_conversion_vdos()))
    assert reporting.create_report.called == 1
    assert is_inhibitor(reporting.create_report.report_fields)
    assert reporting.create_report.report_fields['summary'].startswith('VDO devices')


def test_pre_conversion_single_vdo_incomplete(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(),
                          pre_conversion=_pre_conversion_vdos(1),
                          undetermined_conversion=_undetermined_conversion_vdos()))
    assert reporting.create_report.called == 1
    assert is_inhibitor(reporting.create_report.report_fields)
    assert (reporting.create_report.report_fields['summary'].startswith('VDO device')
            and (not reporting.create_report.report_fields['summary'].startswith('VDO devices')))


# Undetermined tests.
def test_undetermined_single_check_failing(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos(1, True)))
    assert reporting.create_report.called == 1
    assert is_inhibitor(reporting.create_report.report_fields)
    assert (reporting.create_report.report_fields['summary'].startswith(
            'Unexpected result checking device') and
            (not reporting.create_report.report_fields['summary'].startswith(
             'Unexpected result checking devices')))


def test_undetermined_multiple_check_failing(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos(3, failing=True)))
    assert reporting.create_report.called == 1
    assert is_inhibitor(reporting.create_report.report_fields)
    assert reporting.create_report.report_fields['summary'].startswith(
            'Unexpected result checking devices')


def test_undetermined_multiple_no_check(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorCheckVdo())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos(3)))
    assert reporting.create_report.called == 1
    assert is_inhibitor(reporting.create_report.report_fields)
    assert reporting.create_report.report_fields['summary'].startswith(
            'The check of block devices could not be performed as the \'vdo\' '
            'package is not installed.')


# all_vdo_converted test.
def test_all_vdo_converted_true(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', MockedActorAllVdoConvertedTrue())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkvdo.check_vdo(
        VdoConversionInfo(post_conversion=_post_conversion_vdos(),
                          pre_conversion=_pre_conversion_vdos(),
                          undetermined_conversion=_undetermined_conversion_vdos(3)))
    assert reporting.create_report.called == 1
    assert not is_inhibitor(reporting.create_report.report_fields)
    assert reporting.create_report.report_fields['summary'].startswith(
            'User has asserted all VDO devices on the system have been successfully converted')



from leapp.models import Model
from leapp.topics import TransactionTopic


class TransactionCompleted(Model):
    topic = TransactionTopic


class TransactionDryRun(Model):
    topic = TransactionTopic



import re

from leapp.libraries.common.vsftpdutils import (
    get_default_config_hash,
    STRICT_SSL_READ_EOF,
    TCP_WRAPPERS,
    VSFTPD_DEFAULT_CONFIG_PATH
)
from leapp.libraries.stdlib import api

ANONYMOUS_ENABLE = 'anonymous_enable'


class FileOperations(object):
    def read(self, path):
        with open(path, 'r') as f:
            return f.read()

    def write(self, path, content):
        with open(path, 'w') as f:
            f.write(content)


def _replace_in_config(config_lines, option, value):
    res = []
    for line in config_lines:
        if re.match(r'^\s*' + option, line) is None:
            res.append(line)
        else:
            res.append('# Commented out by Leapp:')
            res.append('#' + line)
    if value is not None:
        res.append('# Added by Leapp:')
        res.append('%s=%s' % (option, value))
    return res


def _restore_default_config_file(fileops=FileOperations()):
    try:
        content = fileops.read(VSFTPD_DEFAULT_CONFIG_PATH)
    except IOError as e:
        api.current_logger().warning('Failed to read vsftpd configuration file: %s' % e)
        return
    lines = content.split('\n')
    lines = _replace_in_config(lines, ANONYMOUS_ENABLE, 'YES')
    content = '\n'.join(lines)
    content += '\n'
    fileops.write(VSFTPD_DEFAULT_CONFIG_PATH, content)


def _migrate_config(config, fileops=FileOperations()):
    if not config.tcp_wrappers and config.strict_ssl_read_eof is not None:
        return
    try:
        content = fileops.read(config.path)
    except IOError as e:
        api.current_logger().warning('Failed to read vsftpd configuration file %s: %s'
                                     % (config.path, e))
        return
    lines = content.split('\n')
    if config.tcp_wrappers:
        lines = _replace_in_config(lines, TCP_WRAPPERS, 'NO')
    if config.strict_ssl_read_eof is None:
        lines = _replace_in_config(lines, STRICT_SSL_READ_EOF, 'NO')
    content = '\n'.join(lines)
    content += '\n'
    try:
        fileops.write(config.path, content)
    except IOError as e:
        api.current_logger().warning('Failed to write vsftpd configuration file %s: %s'
                                     % (config.path, e))


def migrate_configs(facts, fileops=FileOperations()):
    if facts.default_config_hash is not None:
        new_hash = get_default_config_hash(read_func=fileops.read)
        # If the default config file was unmodified, it got replaced during the RPM upgrade,
        # so we have to change it back.
        if facts.default_config_hash != new_hash:
            _restore_default_config_file(fileops=fileops)
    for config in facts.configs:
        _migrate_config(config, fileops=fileops)



import os

from leapp.libraries.common.config.version import get_source_major_version
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import FileInfo, TrackedFilesInfoSource

# TODO(pstodulk): make linter happy about this
# common -> Files supposed to be scanned on all system versions.
# '8' (etc..) -> files supposed to be scanned when particular major version of OS is used
TRACKED_FILES = {
    'common': [
        '/etc/pki/tls/openssl.cnf',
    ],
    '8': [
    ],
    '9': [
    ],
}

# TODO(pstodulk)?: introduce possibility to discover files under a dir that
# are not tracked by any rpm or a specified rpm? Currently I have only one
# use case for that in my head, so possibly it will be better to skip a generic
# solution and just introduce a new actor and msg for that (check whether
# actors not owned by our package(s) are present).


def _get_rpm_name(input_file):
    try:
        rpm_names = run(['rpm', '-qf', '--queryformat', r'%{NAME}\n', input_file], split=True)['stdout']
    except CalledProcessError:
        # is not owned by any rpm
        return ''

    if len(rpm_names) > 1:
        # this is very seatbelt; could happen for directories, but we do
        # not expect here directories specified at all. if so, we should
        # provide list instead of string
        api.current_logger().warning(
            'The {} file is owned by multiple rpms: {}.'
            .format(input_file, ', '.join(rpm_names))
        )
    return rpm_names[0]


def is_modified(input_file):
    """
    Return True if checksum has been changed (or removed).

    Ignores mode, user, type, ...
    """
    result = run(['rpm', '-Vf', '--nomtime', input_file], checked=False)
    if not result['exit_code']:
        return False
    status = result['stdout'].split()[0]
    return status == 'missing' or '5' in status


def scan_file(input_file):
    data = {
        'path': input_file,
        'exists': os.path.exists(input_file),
        'rpm_name': _get_rpm_name(input_file),
    }

    if data['rpm_name']:
        data['is_modified'] = is_modified(input_file)
    else:
        # it's not tracked by any rpm at all, so always False
        data['is_modified'] = False

    return FileInfo(**data)


def scan_files(files):
    return [scan_file(fname) for fname in files]


def process():
    files = scan_files(TRACKED_FILES['common'] + TRACKED_FILES.get(get_source_major_version(), []))
    api.produce(TrackedFilesInfoSource(files=files))



from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.config.version import get_source_major_version
from leapp.libraries.stdlib import api, CalledProcessError


def get_leapp_packages():
    """
    Return the list of leapp and leapp-repository rpms that should be preserved
    during the upgrade.

    It's list of packages that should be preserved, not what is really
    installed.

    The snactor RPM doesn't have to be installed, but if so, we have to take
    care about that too as well to prevent broken dnf transaction.
    """
    # TODO: should we set the seatbelt and exclude leapp RPMs from the target
    # system too?
    generic = ['leapp', 'snactor']
    if get_source_major_version() == '7':
        return generic + ['python2-leapp', 'leapp-upgrade-el7toel8']

    return generic + ['python3-leapp', 'leapp-upgrade-el8toel9']


def _strip_split(data, sep, maxsplit=-1):
    """
    Just like str.split(), but remove ambient whitespaces from all items
    """
    return [item.strip() for item in data.split(sep, maxsplit)]


def _get_main_dump(context, disable_plugins):
    """
    Return the dnf configuration dump of main options for the given context.

    Returns the list of lines after the line with "[main]" section
    """

    cmd = ['dnf', 'config-manager', '--dump']

    if disable_plugins:
        for plugin in disable_plugins:
            cmd += ['--disableplugin', plugin]

    try:
        data = context.call(cmd, split=True)['stdout']
    except CalledProcessError as e:
        api.current_logger().error('Cannot obtain the dnf configuration')
        raise StopActorExecutionError(
            message='Cannot obtain data about the DNF configuration',
            details={'stdout': e.stdout, 'stderr': e.stderr}
        )

    try:
        # return index of the first item in the main section
        main_start = data.index('[main]') + 1
    except ValueError:
        raise StopActorExecutionError(
            message='Invalid DNF configuration data (missing [main])',
            details=data,
        )

    output_data = {}
    for line in data[main_start:]:
        try:
            key, val = _strip_split(line, '=', 1)
        except ValueError:
            # This is not expected to happen, but call it a seatbelt in case
            # the dnf dump implementation will change and we will miss it
            # This is not such a hard error as the one above, as it means
            # some values could be incomplete, however we are still able
            # to continue.
            api.current_logger().warning(
                'Cannot parse the dnf dump correctly, line: {}'.format(line))
            pass
        output_data[key] = val

    return output_data


def _get_excluded_pkgs(context, disable_plugins):
    """
    Return the list of excluded packages for DNF in the given context.

    It shouldn't be used on the source system. It is expected this functions
    is called only in the target userspace container or on the target system.
    """
    pkgs = _strip_split(_get_main_dump(context, disable_plugins).get('exclude', ''), ',')
    return [i for i in pkgs if i]


def _set_excluded_pkgs(context, pkglist, disable_plugins):
    """
    Configure DNF to exclude packages in the given list

    Raise the CalledProcessError on error.
    """
    exclude = 'exclude={}'.format(','.join(pkglist))
    cmd = ['dnf', 'config-manager', '--save', '--setopt', exclude]

    if disable_plugins:
        for plugin in disable_plugins:
            cmd += ['--disableplugin', plugin]

    try:
        context.call(cmd)
    except CalledProcessError:
        api.current_logger().error('Cannot set the dnf configuration')
        raise
    api.current_logger().debug('The DNF configuration has been updated to exclude leapp packages.')


def exclude_leapp_rpms(context, disable_plugins):
    """
    Ensure the leapp RPMs are excluded from any DNF transaction.

    This has to be called several times to ensure that our RPMs are not removed
    or updated (replaced) during the IPU. The action should happen inside
        - the target userspace container
        - on the host system
    So user will have to drop these packages from the exclude after the
    upgrade.
    """
    to_exclude = list(set(_get_excluded_pkgs(context, disable_plugins) + get_leapp_packages()))
    _set_excluded_pkgs(context, to_exclude, disable_plugins)



from leapp.actors import Actor
from leapp.libraries.actor import userspacegen
from leapp.libraries.common.config import get_env, version
from leapp.models import RequiredTargetUserspacePackages  # deprecated
from leapp.models import TMPTargetRepositoriesFacts  # deprecated
from leapp.models import (
    CustomTargetRepositoryFile,
    PkgManagerInfo,
    Report,
    RepositoriesFacts,
    RepositoriesMapping,
    RHSMInfo,
    RHUIInfo,
    StorageInfo,
    TargetOSInstallationImage,
    TargetRepositories,
    TargetUserSpaceInfo,
    TargetUserSpacePreupgradeTasks,
    UsedTargetRepositories,
    XFSPresence
)
from leapp.tags import IPUWorkflowTag, TargetTransactionFactsPhaseTag


# @suppress_deprecation(RequiredTargetUserspacePackages, TMPTargetRepositoriesFacts)
class TargetUserspaceCreator(Actor):
    """
    Initializes a directory to be populated as a minimal environment to run binaries from the target system.

    The target userspace is set up in a directory so one can run it in a containerized environment to perform tasks
    as if the system running would be the target system. This allows us to use the target system RPM stack including
    DNF which gives us the ability to use RPM features from the target system.
    The userspace environment is also used to generate a initram disk with dracut using the target system binaries
    such as kernel, systemd etc etc
    """

    name = 'target_userspace_creator'
    consumes = (
        CustomTargetRepositoryFile,
        RHSMInfo,
        RHUIInfo,
        RepositoriesFacts,
        RepositoriesMapping,
        RequiredTargetUserspacePackages,
        StorageInfo,
        TargetOSInstallationImage,
        TargetRepositories,
        TargetUserSpacePreupgradeTasks,
        XFSPresence,
        PkgManagerInfo,
    )
    produces = (TargetUserSpaceInfo, UsedTargetRepositories, Report, TMPTargetRepositoriesFacts,)
    tags = (IPUWorkflowTag, TargetTransactionFactsPhaseTag)

    def process(self):
        skip_check = get_env('LEAPP_DEVEL_SKIP_CHECK_OS_RELEASE', False)
        if (skip_check or version.is_supported_version()) and next(self.consume(RepositoriesMapping), None):
            userspacegen.perform()



import warnings

import pytest

from leapp.libraries.actor.systemfacts import get_selinux_status
from leapp.models import SELinuxFacts

no_selinux = False
try:
    import selinux
except ImportError:
    no_selinux = True
    warnings.warn(
        'Tests which uses `selinux` will be skipped'
        ' due to library unavailability.', ImportWarning)


reason_to_skip_msg = "Selinux is not available"

# FIXME: create valid tests...


@pytest.mark.skipif(no_selinux, reason=reason_to_skip_msg)
def test_selinux_enabled_enforcing(monkeypatch):
    """
    Test case SELinux is enabled in enforcing mode
    """
    monkeypatch.setattr(selinux, 'is_selinux_mls_enabled', lambda: 1)
    monkeypatch.setattr(selinux, 'security_getenforce', lambda: 1)
    monkeypatch.setattr(selinux, 'selinux_getenforcemode', lambda: [0, 1])
    monkeypatch.setattr(selinux, 'is_selinux_enabled', lambda: 1)
    monkeypatch.setattr(selinux, 'selinux_getpolicytype', lambda: [0, 'targeted'])
    expected_data = {'policy': 'targeted',
                     'mls_enabled': True,
                     'enabled': True,
                     'runtime_mode': 'enforcing',
                     'static_mode': 'enforcing'}
    assert SELinuxFacts(**expected_data) == get_selinux_status()


@pytest.mark.skipif(no_selinux, reason=reason_to_skip_msg)
def test_selinux_enabled_permissive(monkeypatch):
    """
    Test case SELinux is enabled in permissive mode
    """
    monkeypatch.setattr(selinux, 'is_selinux_mls_enabled', lambda: 1)
    monkeypatch.setattr(selinux, 'security_getenforce', lambda: 0)
    monkeypatch.setattr(selinux, 'selinux_getenforcemode', lambda: [0, 0])
    monkeypatch.setattr(selinux, 'is_selinux_enabled', lambda: 1)
    monkeypatch.setattr(selinux, 'selinux_getpolicytype', lambda: [0, 'targeted'])
    expected_data = {'policy': 'targeted',
                     'mls_enabled': True,
                     'enabled': True,
                     'runtime_mode': 'permissive',
                     'static_mode': 'permissive'}
    assert SELinuxFacts(**expected_data) == get_selinux_status()


@pytest.mark.skipif(no_selinux, reason=reason_to_skip_msg)
def test_selinux_disabled(monkeypatch):
    """
    Test case SELinux is disabled
    """
    monkeypatch.setattr(selinux, 'is_selinux_mls_enabled', lambda: 0)
    monkeypatch.setattr(selinux, 'security_getenforce', lambda: 0)
    monkeypatch.setattr(selinux, 'selinux_getenforcemode', lambda: [0, 0])
    monkeypatch.setattr(selinux, 'is_selinux_enabled', lambda: 0)
    monkeypatch.setattr(selinux, 'selinux_getpolicytype', lambda: [0, 'targeted'])
    expected_data = {'policy': 'targeted',
                     'mls_enabled': False,
                     'enabled': False,
                     'runtime_mode': 'permissive',
                     'static_mode': 'permissive'}
    assert SELinuxFacts(**expected_data) == get_selinux_status()


class MockNoConfigFileOSError(object):
    def __init__(self):
        raise OSError


@pytest.mark.skipif(no_selinux, reason=reason_to_skip_msg)
def test_selinux_disabled_no_config_file(monkeypatch):
    """
    Test case SELinux is disabled
    """
    monkeypatch.setattr(selinux, 'is_selinux_mls_enabled', lambda: 0)
    monkeypatch.setattr(selinux, 'security_getenforce', lambda: 0)
    monkeypatch.setattr(selinux, 'selinux_getenforcemode', MockNoConfigFileOSError)
    monkeypatch.setattr(selinux, 'is_selinux_enabled', lambda: 0)
    monkeypatch.setattr(selinux, 'selinux_getpolicytype', lambda: [0, 'targeted'])
    expected_data = {'policy': 'targeted',
                     'mls_enabled': False,
                     'enabled': False,
                     'runtime_mode': 'permissive',
                     'static_mode': 'disabled'}

    assert SELinuxFacts(**expected_data) == get_selinux_status()



import os
import shutil

from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.actor import selinuxapplycustom
from leapp.libraries.actor.selinuxapplycustom import BACKUP_DIRECTORY
from leapp.libraries.stdlib import CalledProcessError, run
from leapp.models import SELinuxCustom, SELinuxModules
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag

WORKING_DIRECTORY = '/tmp/selinux/'


class SELinuxApplyCustom(Actor):
    """
    Re-apply SELinux customizations from the original RHEL installation

    Re-apply SELinux policy customizations (custom policy modules and changes
    introduced by semanage). Any changes (due to incompatibility with
    SELinux policy in the upgraded system) are reported to user.
    """
    name = 'selinuxapplycustom'
    consumes = (SELinuxCustom, SELinuxModules)
    produces = (reporting.Report,)
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)

    def process(self):
        # save progress for repoting purposes
        failed_modules = []
        failed_custom = []

        # clear working directory
        shutil.rmtree(WORKING_DIRECTORY, ignore_errors=True)

        try:
            os.mkdir(WORKING_DIRECTORY)
        except OSError:
            self.log.warning('Failed to create working directory! Aborting.')
            return

        # get list of policy modules after the upgrade
        installed_modules = set(
            [module[0] for module in selinuxapplycustom.list_selinux_modules()]
        )

        # import custom SElinux modules
        for semodules in self.consume(SELinuxModules):
            self.log.info(
                'Processing custom SELinux policy modules. Count: {}.'.format(len(semodules.modules))
            )
            # check for presence of udica templates and make sure to install their latest versions
            selinuxapplycustom.install_udica_templates(semodules.templates)

            if not semodules.modules:
                continue

            command = ['semodule']
            for module in semodules.modules:
                # Skip modules that are already installed. This prevents DSP modules installed with wrong
                # priority (usually 400) from being overwritten by an older version
                if module.name in installed_modules:
                    self.log.info(
                        'Skipping module {} on priority {} because it is already installed.'.format(
                            module.name,
                            module.priority
                        )
                    )
                    continue

                # cil module files need to be extracted to disk in order to be installed
                cil_filename = os.path.join(
                    WORKING_DIRECTORY, '{}.cil'.format(module.name)
                )
                self.log.info(
                    'Installing module {} on priority {}.'.format(module.name, module.priority)
                )
                if module.removed:
                    self.log.warning(
                        '{}: The following lines where removed because of incompatibility:\n{}'.format(
                            module.name,
                            '\n'.join(module.removed)
                        )
                    )
                # write module content to disk
                try:
                    with open(cil_filename, 'w') as cil_file:
                        cil_file.write(module.content)
                except OSError as e:
                    self.log.warning('Error writing {} : {}'.format(cil_filename, e))
                    continue

                command.extend(['-X', str(module.priority), '-i', cil_filename])

            try:
                run(command)
            except CalledProcessError as e:
                self.log.warning(
                    'Error installing modules in a single transaction:'
                    '{}\nRetrying -- now each module will be installed separately.'.format(e.stderr)
                )
                # Retry, but install each module separately
                for module in semodules.modules:
                    if module.name in installed_modules:
                        continue
                    cil_filename = os.path.join(
                        WORKING_DIRECTORY, '{}.cil'.format(module.name)
                    )
                    self.log.info(
                        'Installing module {} on priority {}.'.format(module.name, module.priority)
                    )
                    try:
                        run(['semodule',
                             '-X', str(module.priority),
                             '-i', cil_filename
                             ]
                            )
                    except CalledProcessError as e:
                        self.log.warning('Error installing module: {}'.format(e.stderr))
                        failed_modules.append(module.name)
                        selinuxapplycustom.back_up_failed(cil_filename)
                        continue

        # import SELinux customizations collected by "semanage export"
        for custom in self.consume(SELinuxCustom):
            self.log.info(
                'Importing the following SELinux customizations collected by "semanage export":\n{}'.format(
                    '\n'.join(custom.commands)
                )
            )
            # import customizations
            try:
                run(['semanage', 'import'], stdin='\n'.join(custom.commands))
            except CalledProcessError as e:
                self.log.warning(
                    'Error importing SELinux customizations in a single transaction:'
                    '{}\nRetrying -- now each command will be applied separately.'.format(e.stderr)
                )
                for cmd in custom.commands:
                    try:
                        run(['semanage', 'import'], stdin='{}\n'.format(cmd))
                    except CalledProcessError as e:
                        self.log.warning('Error applying "semanage {}": {}'.format(cmd, e.stderr))
                        # retry with "-m" instead of -a
                        cmd_m = selinuxapplycustom.modify_instead_of_add(cmd)
                        if cmd_m:
                            try:
                                run(['semanage', 'import'], stdin='{}\n'.format(cmd_m))
                            except CalledProcessError as e:
                                self.log.warning('Error applying "semanage {}": {}'.format(cmd_m, e.stderr))
                                failed_custom.append(cmd)
                        else:
                            failed_custom.append(cmd)
                continue

        # clean-up
        shutil.rmtree(WORKING_DIRECTORY, ignore_errors=True)

        if failed_modules or failed_custom:
            summary = ''
            if failed_modules:
                summary = (
                    'The following policy modules couldn\'t be installed: {}.\n'
                    'You can review their content in {}.'.format(
                        ', '.join(failed_modules), BACKUP_DIRECTORY
                    )
                )
            if failed_custom:
                if summary:
                    summary = '{}\n\n'.format(summary)
                summary = '{}The following commands couldn\'t be applied:\n{}'.format(
                    summary, '\n'.join(['semanage {}'.format(x) for x in failed_custom])
                )

            reporting.create_report(
                [
                    reporting.Title(
                        'SELinux failed to reapply some customizations after the upgrade.'
                    ),
                    reporting.Summary(summary),
                    reporting.Severity(reporting.Severity.MEDIUM),
                    reporting.Groups([reporting.Groups.SECURITY, reporting.Groups.SELINUX]),
                ]
                + [
                    reporting.RelatedResource(
                        'file', os.path.join(BACKUP_DIRECTORY, '{}.cil'.format(x))
                    )
                    for x in failed_modules
                ]
            )



import os

from leapp.libraries.common.config import mock_configs
from leapp.models import (
    DNFWorkaround,
    InstalledRPM,
    Module,
    RepositoriesSetupTasks,
    RPM,
    RpmTransactionTasks,
    SatelliteFacts
)
from leapp.snactor.fixture import current_actor_context

RH_PACKAGER = 'Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>'


def fake_package(pkg_name):
    return RPM(name=pkg_name, version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
               pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51')


FOREMAN_RPM = fake_package('foreman')
FOREMAN_PROXY_RPM = fake_package('foreman-proxy')
KATELLO_INSTALLER_RPM = fake_package('foreman-installer-katello')
KATELLO_RPM = fake_package('katello')
POSTGRESQL_RPM = fake_package('rh-postgresql12-postgresql-server')
SATELLITE_RPM = fake_package('satellite')
SATELLITE_CAPSULE_RPM = fake_package('satellite-capsule')


def test_no_satellite_present(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(SatelliteFacts)
    assert not message


def test_satellite_present(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(SatelliteFacts)[0]
    assert message.has_foreman


def test_wrong_arch(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG_S390X)
    message = current_actor_context.consume(SatelliteFacts)
    assert not message


def test_satellite_capsule_present(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_PROXY_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(SatelliteFacts)[0]
    assert message.has_foreman


def test_no_katello_installer_present(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(SatelliteFacts)[0]
    assert not message.has_katello_installer


def test_katello_installer_present(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM, KATELLO_INSTALLER_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(SatelliteFacts)[0]
    assert message.has_katello_installer


def test_enables_ruby_module(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(RpmTransactionTasks)[0]
    assert Module(name='ruby', stream='2.7') in message.modules_to_enable


def test_enables_pki_modules(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM, KATELLO_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(RpmTransactionTasks)[0]
    assert Module(name='pki-core', stream='10.6') in message.modules_to_enable
    assert Module(name='pki-deps', stream='10.6') in message.modules_to_enable


def test_enables_satellite_module(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM, SATELLITE_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(RpmTransactionTasks)[0]
    assert Module(name='satellite', stream='el8') in message.modules_to_enable
    assert Module(name='satellite-capsule', stream='el8') not in message.modules_to_enable


def test_enables_satellite_capsule_module(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_PROXY_RPM, SATELLITE_CAPSULE_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(RpmTransactionTasks)[0]
    assert Module(name='satellite-capsule', stream='el8') in message.modules_to_enable
    assert Module(name='satellite', stream='el8') not in message.modules_to_enable


def test_installs_satellite_package(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM, SATELLITE_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(RpmTransactionTasks)[0]
    assert 'satellite' in message.to_install
    assert 'satellite-capsule' not in message.to_install


def test_installs_satellite_capsule_package(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_PROXY_RPM, SATELLITE_CAPSULE_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    message = current_actor_context.consume(RpmTransactionTasks)[0]
    assert 'satellite-capsule' in message.to_install
    assert 'satellite' not in message.to_install


def test_detects_local_postgresql(monkeypatch, current_actor_context):
    def mock_stat():
        orig_stat = os.stat

        def mocked_stat(path):
            if path == '/var/opt/rh/rh-postgresql12/lib/pgsql/data/':
                path = '/'
            return orig_stat(path)
        return mocked_stat
    monkeypatch.setattr("os.stat", mock_stat())

    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM, POSTGRESQL_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)

    rpmmessage = current_actor_context.consume(RpmTransactionTasks)[0]
    assert Module(name='postgresql', stream='12') in rpmmessage.modules_to_enable

    satellitemsg = current_actor_context.consume(SatelliteFacts)[0]
    assert satellitemsg.postgresql.local_postgresql

    assert current_actor_context.consume(DNFWorkaround)


def test_detects_remote_postgresql(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)

    rpmmessage = current_actor_context.consume(RpmTransactionTasks)[0]
    assert Module(name='postgresql', stream='12') not in rpmmessage.modules_to_enable

    satellitemsg = current_actor_context.consume(SatelliteFacts)[0]
    assert not satellitemsg.postgresql.local_postgresql

    assert not current_actor_context.consume(DNFWorkaround)


def test_enables_right_repositories_on_satellite(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_RPM, SATELLITE_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)

    rpmmessage = current_actor_context.consume(RepositoriesSetupTasks)[0]

    assert 'satellite-maintenance-6.11-for-rhel-8-x86_64-rpms' in rpmmessage.to_enable
    assert 'satellite-6.11-for-rhel-8-x86_64-rpms' in rpmmessage.to_enable
    assert 'satellite-capsule-6.11-for-rhel-8-x86_64-rpms' not in rpmmessage.to_enable


def test_enables_right_repositories_on_capsule(current_actor_context):
    current_actor_context.feed(InstalledRPM(items=[FOREMAN_PROXY_RPM, SATELLITE_CAPSULE_RPM]))
    current_actor_context.run(config_model=mock_configs.CONFIG)

    rpmmessage = current_actor_context.consume(RepositoriesSetupTasks)[0]

    assert 'satellite-maintenance-6.11-for-rhel-8-x86_64-rpms' in rpmmessage.to_enable
    assert 'satellite-6.11-for-rhel-8-x86_64-rpms' not in rpmmessage.to_enable
    assert 'satellite-capsule-6.11-for-rhel-8-x86_64-rpms' in rpmmessage.to_enable



import pytest

from leapp import reporting
from leapp.libraries.actor import checkmicroarchitecture
from leapp.libraries.common.config.architecture import ARCH_SUPPORTED, ARCH_X86_64
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked, logger_mocked
from leapp.libraries.stdlib import api
from leapp.models import CPUInfo
from leapp.utils.report import is_inhibitor


@pytest.mark.parametrize("arch", [arch for arch in ARCH_SUPPORTED if not arch == ARCH_X86_64])
def test_not_x86_64_passes(monkeypatch, arch):
    """
    Test no report is generated on an architecture different from x86-64
    """

    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(arch=arch))

    checkmicroarchitecture.process()

    assert 'Architecture not x86-64. Skipping microarchitecture test.' in api.current_logger.infomsg
    assert not reporting.create_report.called


def test_valid_microarchitecture(monkeypatch):
    """
    Test no report is generated on a valid microarchitecture
    """

    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    required_flags = checkmicroarchitecture.X86_64_BASELINE_FLAGS + checkmicroarchitecture.X86_64_V2_FLAGS
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(arch=ARCH_X86_64,
                                                                 msgs=[CPUInfo(flags=required_flags)]))

    checkmicroarchitecture.process()

    assert 'Architecture not x86-64. Skipping microarchitecture test.' not in api.current_logger.infomsg
    assert not reporting.create_report.called


def test_invalid_microarchitecture(monkeypatch):
    """
    Test report is generated on x86-64 architecture with invalid microarchitecture and the upgrade is inhibited
    """

    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(arch=ARCH_X86_64, msgs=[CPUInfo()]))

    checkmicroarchitecture.process()

    produced_title = reporting.create_report.report_fields.get('title')
    produced_summary = reporting.create_report.report_fields.get('summary')

    assert 'Architecture not x86-64. Skipping microarchitecture test.' not in api.current_logger().infomsg
    assert reporting.create_report.called == 1
    assert 'microarchitecture is unsupported' in produced_title
    assert 'RHEL9 has a higher CPU requirement' in produced_summary
    assert reporting.create_report.report_fields['severity'] == reporting.Severity.HIGH
    assert is_inhibitor(reporting.create_report.report_fields)
